<?php

class Entity_Form_EmployeeApply {

  const FIELD_TYPE_TEXT = 'text';
  const FIELD_TYPE_PASSWORD = 'password';
  const FIELD_TYPE_SYSTEM = 'system';
  const FIELD_TYPE_CHECK = 'check';
  const FIELD_TYPE_GROUP = 'group';
  const FIELD_TYPE_DATE = 'date';
  const FIELD_TYPE_FILE = 'file';
  const FIELD_TYPE_LABEL = 'label';
  const DOCUMENT_CV_ATTACHMENT = 'CVTag';
  const DOCUMENT_PHOTO_TAG = 'PhotoTag';
  const DOCUMENT_EXTRA_TAG = 'DocumentUploadedByCandidateTag';
  const FIELD_TYPE_TEXTAREA = 'textarea';

  static $DISPLAYED = false;
  static $employee = null;
  static $user = null;
  static $emails = array();
  static $fields = array(
    // <editor-fold defaultstate="collapsed" desc="Personal section">
    'personal' => array(
      'type' => 'individual',
      'label' => 'CxPortal.message.text.personalData',
      'items' => array(
        'lastName' => array(
          'label' => 'general.message.field.lastName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'lastName',
          'rule' => 'max_chars:60'
        ),
        'middleName' => array(
          'label' => 'CxPortal.message.field.middleName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'lastNamePrefix',
          'rule' => 'max_chars:40'
        ),
        'lastNameAtBirth' => array(
          'label' => 'general.message.field.lastNameBirth',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'birthName',
          'rule' => 'max_chars:40'
        ),
        'initials' => array(
          'label' => 'general.message.field.initials',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'initials',
          'rule' => 'max_chars:12'
        ),
        'firstName' => array(
          'label' => 'general.message.field.firstName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'firstName',
          'rule' => 'max_chars:40'
        ),
        'title' => array(
          'label' => 'general.message.field.title',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'title',
          'rule' => 'max_chars:32'
        ),
        'suffix' => array(
          'label' => 'general.message.field.suffix',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'suffix',
          'rule' => 'max_chars:12'
        ),
        'password' => array(
          'label' => 'general.message.field.password',
          'type' => self::FIELD_TYPE_PASSWORD,
          'mapsTo' => 'password',
        ),
        'gender' => array(
          'label' => 'general.message.field.gender',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'genderNode',
          'system_table' => 'Salutation'
        ),
        'maritalStatus' => array(
          'label' => 'general.message.field.maritalStatus',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'maritalStatusNode',
          'system_table' => 'Marital Status',
        ),
        'birthDate' => array(
          'label' => 'general.message.field.birthDate',
          'type' => self::FIELD_TYPE_DATE,
          'mapsTo' => 'birthDate',
          'rule' => 'is_date:1'
        ),
        'birthPlace' => array(
          'label' => 'general.message.field.birthCity',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'birthCity',
          'rule' => 'max_chars:40'
        ),
        'countryOfBirth' => array(
          'label' => 'general.message.field.birthCountry',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'birthCountryNode',
          'system_table' => 'Country',
        ),
        'nationality' => array(
          'label' => 'general.message.field.nationality',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'nationalityNode',
          'system_table' => 'Nationality',
        ),
        'functionLevel' => array(
          'label' => 'general.message.field.functionLevel',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'WorkLevelNode',
          'system_table' => 'Professional level',
        ),
        /*
          'experience' => array(
          'label' => 'general.message.field.experience',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'ExperienceNode',experienceSinceDate
          'system_table' => 'Experience',
          ), */
        'experience' => array(
          'label' => 'general.message.field.experience',
          'type' => self::FIELD_TYPE_DATE,
          'mapsTo' => 'experienceSinceDate',
        ),
        'cv' => array(
          'label' => 'CxPortal.message.field.cv1',
          'type' => self::FIELD_TYPE_FILE,
          'mapsTo' => 'cv',
        ), // [todo]
        'photo' => array(
          'label' => 'CxPortal.message.field.photo',
          'type' => self::FIELD_TYPE_FILE,
          'mapsTo' => 'photo',
        ), // [todo]
        'coverLetter' => array(
          'label' => 'CxPortal.message.field.coverLetter',
          'type' => self::FIELD_TYPE_FILE,
          'mapsTo' => 'coverLetter',
          'repeatable' => true,
        ), // [todo]
        'hobbies' => array(
            'label' => 'CxPortal.message.field.hobbies',
            'type'  => self::FIELD_TYPE_TEXTAREA,
            'mapsTo'=> 'hobbies'
        )
      ),
    ), // personal section
    // </editor-fold>
    //<editor-fold defaultstate="collapsed" desc="Contact details">
    'contact' => array(
      'type' => 'individual',
      'label' => 'CxPortal.header.contactDetails',
      'items' => array(
        'email' => array(
          'label' => 'general.message.field.email',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'EmailAddress',
          'systemtype' => 'E-mailadres-label',
          'repeatable' => true,
          'rule' => 'max_chars:200',
          'html_type' => 'email'
        ),
        'homePhone' => array(
          'label' => 'general.message.field.homePhone',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'PhoneNumber',
          'rule' => 'max_chars:40'
        ),
        'mobileNumber' => array(
          'label' => 'general.message.field.mobilePrivate',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'mobileNumber',
          'rule' => 'max_chars:40'
        ),
        'mobileNumberBusiness' => array(
          'label' => 'general.message.field.mobileWork',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'mobileNumberBusiness',
          'rule' => 'max_chars:40'
        ),
        'workPhone' => array(
          'label' => 'general.message.field.workPhone',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'PhoneNumberBusiness',
          'rule' => 'max_chars:40'
        ),
        'contactInstructions' => array(
          'label' => 'general.message.field.contactInstructions',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'ContactInfo',
          'rule' => 'max_chars:80'
        ),
      ),
    ), // contact details section
    //</editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Address">
    'address' => array(
      'type' => 'collection',
      'label' => 'general.header.address',
      // warning, this item is possible to change later on, depending on the chosen address format
      'items' => array(
        'homeAddressLabel' => array(
          'label' => 'CxPortal.header.addressPrivate',
          'type' => self::FIELD_TYPE_LABEL,
          'visibility' => 'container',
        ),
        'homeStreetName' => array(
          'label' => 'general.message.field.streetName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'HomeStreet',
          'visibility_linked' => 'homeAddressLabel',
          'rule' => 'max_chars:60'
        ),
        'homeStreetNumber' => array(
          'label' => 'general.message.field.streetNo',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'HomeNumber',
          'visibility_linked' => 'homeAddressLabel',
          'rule' => 'max_chars:10'
        ),
        'homeSuffixNumber' => array(
          'label' => 'general.message.field.adding',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'HomeNumberSuffix',
          'visibility_linked' => 'homeAddressLabel',
          'rule' => 'max_chars:10'
        ),
        'homeCity' => array(
          'label' => 'general.message.field.city',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'homeCity',
          'visibility_linked' => 'homeAddressLabel',
          'rule' => 'max_chars:40'
        ),
        'homePostalCode' => array(
          'label' => 'general.message.field.zipCode',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'HomePostalCode',
          'visibility_linked' => 'homeAddressLabel',
          'rule' => 'max_chars:30'
        ),
        'homeCountry' => array(
          'label' => 'general.message.field.country',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'HomeCountryNode',
          'system_table' => 'Country',
          'visibility_linked' => 'homeAddressLabel',
          'affects' => 'homeRegion',
        ),
        'homeRegion' => array(
          'label' => 'general.message.field.region',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'HomeProvinceNode',
          'system_table' => 'Region',
          'visibility_linked' => 'homeAddressLabel',
          'affectable' => true,
          'affected_by' => 'homeCountry',
        ), // end subsection home address
        'workAddressLabel' => array(
          'label' => 'CxPortal.header.addressWork',
          'type' => self::FIELD_TYPE_LABEL,
          'visibility' => 'container',
        ),
        'workStreetName' => array(
          'label' => 'general.message.field.streetName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'visitStreet',
          'visibility_linked' => 'workAddressLabel',
          'rule' => 'max_chars:40'
        ),
        'workStreetNumber' => array(
          'label' => 'general.message.field.streetNo',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'visitNumber',
          'visibility_linked' => 'workAddressLabel',
          'rule' => 'max_chars:10'
        ),
        'workSuffixNumber' => array(
          'label' => 'general.message.field.adding',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'VisitNumberSuffix',
          'visibility_linked' => 'workAddressLabel',
          'rule' => 'max_chars:10'
        ),
        'workCity' => array(
          'label' => 'general.message.field.city',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'visitCity',
          'visibility_linked' => 'workAddressLabel',
          'rule' => 'max_chars:40'
        ),
        'workPostalCode' => array(
          'label' => 'general.message.field.zipCode',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'visitPostalCode',
          'visibility_linked' => 'workAddressLabel',
          'rule' => 'max_chars:30'
        ),
        'workCountry' => array(
          'label' => 'general.message.field.country',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'VisitCountryNode',
          'system_table' => 'Country',
          'visibility_linked' => 'workAddressLabel',
          'affects' => 'workRegion'
        ),
        'workRegion' => array(
          'label' => 'general.message.field.region',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'VisitProvinceNode',
          'system_table' => 'Region',
          'visibility_linked' => 'workAddressLabel',
          'affectable' => true,
          'affected_by' => 'workCountry'
        ), // end work subsection
        'alternativeAddressLabel' => array(
          'label' => 'CxPortal.header.addressAlternative',
          'type' => self::FIELD_TYPE_LABEL,
          'visibility' => 'container',
        ),
        'alternativeStreetName' => array(
          'label' => 'general.message.field.streetName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'alternativeStreet',
          'visibility_linked' => 'alternativeAddressLabel',
          'rule' => 'max_chars:40'
        ),
        'alternativeStreetNumber' => array(
          'label' => 'general.message.field.streetNo',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'alternativeNumber',
          'visibility_linked' => 'alternativeAddressLabel',
          'rule' => 'max_chars:10'
        ),
        'alternativeSuffixNumber' => array(
          'label' => 'general.message.field.adding',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'AlternativeNumberSuffix',
          'visibility_linked' => 'alternativeAddressLabel',
          'rule' => 'max_chars:10'
        ),
        'alternativeCity' => array(
          'label' => 'general.message.field.city',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'alternativeCity',
          'visibility_linked' => 'alternativeAddressLabel',
          'rule' => 'max_chars:40'
        ),
        'alternativePostalCode' => array(
          'label' => 'general.message.field.zipCode',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'alternativePostalCode',
          'visibility_linked' => 'alternativeAddressLabel',
          'rule' => 'max_chars:30'
        ),
        'alternativeCountry' => array(
          'label' => 'general.message.field.country',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'AlternativeCountryNode',
          'system_table' => 'Country',
          'visibility_linked' => 'alternativeAddressLabel',
          'affects' => 'alternativeRegion'
        ),
        'alternativeRegion' => array(
          'label' => 'general.message.field.region',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'AlternativeProvinceNode',
          'system_table' => 'Region',
          'visibility_linked' => 'alternativeAddressLabel',
          'affectable' => true,
          'affected_by' => 'alternativeCountry'
        ), // end alternative address subsection
      ),
    ), // end address details section
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Wishes">
    // start wishes section
    'wishes' => array(
      'type' => 'individual',
      'label' => 'CxPortal.header.wishes',
      'items' => array(
        'employment' => array(
          'label' => 'CxPortal.message.field.employment',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefContractTypeNode',
          'system_table' => 'Employment',
        ),
        'hoursPerWeek' => array(
          'label' => 'CxPortal.message.field.weekWorkHours',
          'hint' => '',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'MaxFte',
          'rule' => 'is_numeric:1',
          'html_type' => 'number'
        ),
        'minSalary' => array(
            'label' => 'general.message.field.salary',
            'type' => self::FIELD_TYPE_TEXT,
            'mapsTo' => 'minSalary',
            'rule' => 'is_numeric:1',
            'html_type' => 'number',
            'related' => array('minSalaryCurrencyNode', 'minSalaryUnitNode'),
        ), 
        'minSalaryCurrencyNode' => array(
            'label' => '',
            'mapsTo' => 'minSalaryCurrencyNode',
            'system_table' => 'Currency',
            'affected_by' => 'minSalary'
        ),
        'minSalaryUnitNode' => array(
            'label' => '',
            'mapsTo' => 'minSalaryUnitNode',
            'system_table' => 'Inkooptarief-eenheid',
            'affected_by' => 'minSalary'
        ),
        // [todo]
        'desiredFunction1' => array(
          'label' => 'CxPortal.message.field.toLevel1Function1',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'ProductTypeNode1',
          'system_table' => 'Business line',
          'affects' => 'desiredFunction1_function0',
        ),
        'desiredFunction1_function0' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'Level1Function1',
          'system_table' => 'Function0',
          'affects' => 'desiredFunction1_function1',
          'affectable' => true,
          'affected_by' => 'desiredFunction1'
        ),
        'desiredFunction1_function1' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'Level2Function1',
          'system_table' => 'Function1',
          'affectable' => true,
          'affected_by' => 'desiredFunction1_function0'
        ),
        'desiredFunction2' => array(
          'label' => 'CxPortal.message.field.toLevel1Function2',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'ProductTypeNode2',
          'system_table' => 'Business line',
          'affects' => 'desiredFunction2_function0',
        ),
        'desiredFunction2_function0' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'Level1Function2',
          'system_table' => 'Function0',
          'affects' => 'desiredFunction2_function1',
          'affectable' => true,
          'affected_by' => 'desiredFunction2'
        ),
        'desiredFunction2_function1' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'Level2Function2',
          'system_table' => 'Function1',
          'affectable' => true,
          'affected_by' => 'desiredFunction2_function0'
        ),
        'desiredFunction3' => array(
          'label' => 'CxPortal.message.field.toLevel1Function3',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'ProductTypeNode3',
          'system_table' => 'Business line',
          'affects' => 'desiredFunction3_function0',
        ),
        'desiredFunction3_function0' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'Level1Function3',
          'system_table' => 'Function0',
          'affects' => 'desiredFunction3_function1',
          'affectable' => true,
          'affected_by' => 'desiredFunction3'
        ),
        'desiredFunction3_function1' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'Level2Function3',
          'system_table' => 'Function1',
          'affectable' => true,
          'affected_by' => 'desiredFunction3_function0'
        ),
        'prefBranche1Level1Node' => array(
          'label' => 'CxPortal.message.field.toBranche1Level1Node',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'prefBranche1Level1Node',
          'system_table' => 'Branch0',
          'affects' => 'prefBranche1Level2Node',
        ),
        'prefBranche1Level2Node' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'prefBranche1Level2Node',
          'system_table' => 'Branch1',
          'affectable' => true,
          'affected_by' => 'prefBranche1Level1Node'
        ),
        'prefBranche2Level1Node' => array(
          'label' => 'CxPortal.message.field.toBranche2Level1Node',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'prefBranche2Level1Node',
          'system_table' => 'Branch0',
          'affects' => 'prefBranche2Level2Node',
        ),
        'prefBranche2Level2Node' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'prefBranche2Level2Node',
          'system_table' => 'Branch1',
          'affectable' => true,
          'affected_by' => 'prefBranche2Level1Node'
        ),
        'prefBranche3Level1Node' => array(
          'label' => 'CxPortal.message.field.toBranche3Level1Node',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'prefBranche3Level1Node',
          'system_table' => 'Branch0',
          'affects' => 'prefBranche3Level2Node',
        ),
        'prefBranche3Level2Node' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'prefBranche3Level2Node',
          'system_table' => 'Branch1',
          'affectable' => true,
          'affected_by' => 'prefBranche3Level1Node'
        ),
        'desiredCountry_1' => array(
          'label' => 'general.message.field.country',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefCountry1Node',
          'system_table' => 'Country',
          'affects' => 'desiredRegion_1',
        ),
        'desiredRegion_1' => array(
          'label' => 'general.message.field.region',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefRegion1Node',
          'system_table' => 'Region',
          'affectable' => true,
          'affected_by' => 'desiredCountry_1'
        ),
        'desiredCountry_2' => array(
          'label' => 'general.message.field.country',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefCountry2Node',
          'system_table' => 'Country',
          'affects' => 'desiredRegion_2',
        ),
        'desiredRegion_2' => array(
          'label' => 'general.message.field.region',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefRegion2Node',
          'system_table' => 'Region',
          'affectable' => true,
          'affected_by' => 'desiredCountry_2'
        ),
        'desiredCountry_3' => array(
          'label' => 'general.message.field.country',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefCountry3Node',
          'system_table' => 'Country',
          'affects' => 'desiredRegion_3',
        ),
        'desiredRegion_3' => array(
          'label' => 'general.message.field.region',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'PrefRegion3Node',
          'system_table' => 'Region',
          'affectable' => true,
          'affected_by' => 'desiredCountry_3'
        ),
      ),
    ), // end wishes section
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Ambition">
    'ambitionAndAvailability' => array(
      'type' => 'individual',
      'label' => 'CxPortal.header.ambitionAvailability',
      'items' => array(
        'ambition' => array(
          'label' => 'CxPortal.message.field.ambition',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'ambition',
          'rule' => 'max_chars:1000'
        ),
        'availableIn' => array(
          'label' => 'CxPortal.message.field.availableIn',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'AvailableWithinNode',
          'system_table' => 'Notice period',
        ),
        'availableFrom' => array(
          'label' => 'CxPortal.message.field.availableFrom',
          'type' => self::FIELD_TYPE_DATE,
          'mapsTo' => 'AvailableFromDate',
          'rule' => 'is_date:1'
        ),
      ),
    ), // end ambition & availability
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Transportation">
    // transportation section
    'transportation' => array(
      'type' => 'individual',
      'label' => 'CxPortal.header.transportation',
      'items' => array(
        'homeWork' => array(
          'label' => 'CxPortal.message.field.transportHomeWork',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'MeansOfTransportNode',
          'system_table' => 'Means of transportation',
        ),
        'haveCar' => array(
          'label' => 'CxPortal.message.field.transportHasCar',
          'type' => self::FIELD_TYPE_CHECK,
          'mapsTo' => 'hasCar',
        ),
        'license' => array(
          'label' => 'CxPortal.message.field.transportDriveLicence',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'DriverLicenceNode',
          'system_table' => 'Drivers license',
        ),
        'issuingPlace' => array(
          'label' => 'CxPortal.message.field.transportPlaceIssueLicence',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'driversLicenseCity',
          'rule' => 'max_chars:40'
        ),
        'licenceNo' => array(
          'label' => 'CxPortal.message.field.transportationLicenceNo',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'driversLicenseCode',
          'rule' => 'max_chars:20'
        ),
        'numberValidity' => array(
          'label' => 'CxPortal.message.field.transportNumeberValidUntil',
          'type' => self::FIELD_TYPE_DATE,
          'mapsTo' => 'driversLicenseValidUntilDate',
          'rule' => 'is_date:1'
        ),
      ),
    ), // end transportation section
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Work Experience" >
    // start work experience section
    'workExperience' => array(
      'type' => 'group',
      'subtype' => 'individual',
      'label' => 'CxPortal.header.experience',
      'items' => array(
        'employer' => array(
          'label' => 'CxPortal.message.field.employer',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'Employer',
          'rule' => 'max_chars:128'
        ),
        'periodOf' => array(
          'label' => 'CxPortal.message.field.experiencePeriod',
          'type' => self::FIELD_TYPE_DATE,
          'mapsTo' => 'StartDate',
          'rule' => 'is_date:1'
        ),
        'periodUntil' => array(
          'label' => 'CxPortal.message.field.until',
          'type' => self::FIELD_TYPE_DATE,
          'mapsTo' => 'EndDate',
          'rule' => 'is_date:1'
        ),
        'functionName' => array(
          'label' => 'CxPortal.message.field.functionName',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'Function',
          'rule' => 'max_chars:128'
        ),
        'workLocation' => array(
          'label' => 'CxPortal.message.field.workLocation',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'WorkLocation',
          'rule' => 'max_chars:128'
        ),
        'position' => array(
          'label' => 'CxPortal.message.field.position',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'ProductType1Node',
          'system_table' => 'Business line',
          'affects' => 'position_level_1',
        ),
        'position_level_1' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'FunctionLevel1',
          'system_table' => 'Function0',
          'affects' => 'position_level_2',
          'affected_by' => 'position',
          'affectable' => true,
          'affected_by' => 'position',
        ),
        'position_level_2' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'FunctionLevel2',
          'system_table' => 'Function1',
          'affectable' => true,
          'affected_by' => 'position_level_1',
        ),
        'industry' => array(
          'label' => 'CxPortal.message.field.industry',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'BrancheLevel1',
          'system_table' => 'Branch0',
          'affects' => 'sector',
        ),
        'sector' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'BrancheLevel2',
          'system_table' => 'Branch1',
          'affectable' => true,
          'affected_by' => 'industry',
        ),
        'assignment' => array(
          'label' => 'CxPortal.message.field.assignment',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'Info',
          'rule' => 'max_chars:1000000'
        ),
      ),
    ), // end work experience section
    'experienceOther' => array(
      'type' => 'subgroup',
      'label' => '',
      'items' => array(
        'experiencenotes' => array(
          'label' => 'CxPortal.message.field.otherExperience',
          'type' => self::FIELD_TYPE_TEXTAREA,
          'mapsTo' => 'ExperienceInformation',
          'rule' => 'max_chars:1000000'
        ),
      ),
    ),
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Education" >
    // start education section
    'education' => array(
      'type' => 'group',
      'subtype' => 'individual',
      'label' => 'CxPortal.header.educations',
      'items' => array(
        'education' => array(
          'label' => 'CxPortal.header.educations',
          'type' => self::FIELD_TYPE_SYSTEM,
          'system_table' => 'Education0',
          'mapsTo' => 'Level1Education1',
          'affects' => 'education_1',
        ),
        'education_1' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'system_table' => 'Education1',
          'mapsTo' => 'Level1Education2',
          'affects' => 'education_2',
          'affectable' => true,
          'affected_by' => 'education'
        ),
        'education_2' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'system_table' => 'Education2',
          'mapsTo' => 'Level1Education3',
          'affectable' => true,
          'affected_by' => 'education_1'
        ),
        'institute' => array(
          'label' => 'CxPortal.message.field.institute',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'Institute',
          'rule' => 'max_chars:128'
        ),
        'yearStart' => array(
          'label' => 'CxPortal.message.field.yearStart',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'StartDate',
          'rule' => 'is_numeric:1'
        ),
        'yearEnd' => array(
          'label' => 'CxPortal.message.field.toYear',
          'type' => self::FIELD_TYPE_TEXT,
          'mapsTo' => 'EndDate',
          'rule' => 'is_numeric:1'
        ),
      ),
    ),
    'educationOther' => array(
      'type' => 'subgroup',
      'label' => '',
      'items' => array(
        'languagenotes' => array(
          'label' => 'CxPortal.message.field.otherCourses',
          'type' => self::FIELD_TYPE_TEXTAREA,
          'mapsTo' => 'EducationInformation',
          'rule' => 'max_chars:1000000'
        ),
      ),
    ),
    // // end education section
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Skills" >
    // start professional skills section
    'professionalSkills' => array(
      'type' => 'group',
      'subtype' => 'individual',
      'label' => 'CxPortal.header.professionalSkills',
      'items' => array(
        'skill' => array(
          'label' => 'CxPortal.message.field.skill',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'SkillLevel1',
          'system_table' => 'Skill0',
          'affects' => 'skill_1',
        ),
        'skill_1' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'SkillLevel2',
          'system_table' => 'Skill1',
          'affects' => 'skill_2',
          'affectable' => true,
          'affected_by' => 'skill'
        ),
        'skill_2' => array(
          'label' => '',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'SkillLevel3',
          'system_table' => 'Skill2',
          'affectable' => true,
          'affected_by' => 'skill_1'
        ),
      ),
    ),
    'skillsOther' => array(
      'type' => 'subgroup',
      'label' => '',
      'items' => array(
        'skillnotes' => array(
          'label' => 'CxPortal.message.field.otherSkills',
          'type' => self::FIELD_TYPE_TEXTAREA,
          'mapsTo' => 'SkillNotes',
          'rule' => 'max_chars:1000000'
        ),
      ),
    ),
    // // end professional skills section
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Languages" >
    'languages' => array(
      'type' => 'group',
      'subtype' => 'individual',
      'label' => 'CxPortal.header.languages',
      'items' => array(
        'language' => array(
          'label' => 'general.message.field.language',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'language',
          'system_table' => 'Language',
        ),
        'read' => array(
          'label' => 'CxPortal.message.field.read',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'languageReadLevel',
          'system_table' => 'Language skill',
        ),
        'write' => array(
          'label' => 'CxPortal.message.field.write',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'languageWriteLevel',
          'system_table' => 'Language skill',
        ),
        'speak' => array(
          'label' => 'CxPortal.message.field.speak',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'languageSpeakLevel',
          'system_table' => 'Language skill',
        ),
      ),
    ),
    'languagesOther' => array(
      'type' => 'subgroup',
      'label' => '',
      'items' => array(
        'notes' => array(
          'label' => 'CxPortal.message.field.otherLanguages',
          'type' => self::FIELD_TYPE_TEXTAREA,
          'mapsTo' => 'languageNotes',
          'rule' => 'max_chars:1000000'
        ),
      ),
    ),
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Motivation & Source" >
    // start motivation and source section
    'motivationAndSource' => array(
      'type' => 'individual',
      'label' => 'CxPortal.header.motivationSource',
      'items' => array(
        'motivation' => array(
          'label' => 'CxPortal.message.field.motivation',
          'type' => self::FIELD_TYPE_TEXTAREA,
          'mapsTo' => 'motivation',
          'rule' => 'max_chars:1000000'
        ),
        'source' => array(
          'label' => 'CxPortal.message.field.foundThrough',
          'type' => self::FIELD_TYPE_SYSTEM,
          'mapsTo' => 'SourceNode',
          'system_table' => 'Bron-Kandidaat', // Source candidate
        ),
        'sourceOther' => array(
          'label' => 'CxPortal.message.field.other',
          'type' => self::FIELD_TYPE_TEXTAREA,
          'mapsTo' => 'sourceOther',
          'rule' => 'max_chars:1000000'
        ),
      ),
    ), // end motivation and source section
    // </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="Newsletter subscription" >
    'newsletterSubscription' => array(
      'type' => 'individual',
      'label' => 'CxPortal.header.newsletterSubscription',
      'items' => array(
        'subscribe' => array(
          'label' => 'CxPortal.message.field.subscribeNewsletter',
          'type' => self::FIELD_TYPE_CHECK,
        )
      )
    )

      // </editor-fold>
  );
  static $HAS_NEWSLETTER = false;
  static $profSkillsGroupCount = 0;
  static $profLanguagesGroupCount = 0;
  static $educationGroupCount = 0;
  static $workGroupCount = 0;
  static $currentConfig = array();

  static function addressInternationalFormat() {

    return array(
      'homeAddressLabel' => array(
        'label' => 'CxPortal.header.addressPrivate',
        'type' => self::FIELD_TYPE_LABEL,
        'visibility' => 'container',
      ),
      'homeFullAddress' => array(
        'label' => 'general.header.fullAddress',
        'type' => self::FIELD_TYPE_TEXTAREA,
        'mapsTo' => 'fullAddress',
        'visibility_linked' => 'homeAddressLabel',
        'rule' => 'max_chars:255'
      ),
      'homeCountry' => array(
        'label' => 'general.message.field.country',
        'type' => self::FIELD_TYPE_SYSTEM,
        'mapsTo' => 'HomeCountryNode',
        'system_table' => 'Country',
        'visibility_linked' => 'homeAddressLabel',
        'affects' => 'homeRegion',
      ),
      'homeRegion' => array(
        'label' => 'general.message.field.region',
        'type' => self::FIELD_TYPE_SYSTEM,
        'mapsTo' => 'HomeProvinceNode',
        'system_table' => 'Region',
        'visibility_linked' => 'homeAddressLabel',
        'affectable' => true,
      ),
      'workAddressLabel' => array(
        'label' => 'CxPortal.header.addressWork',
        'type' => self::FIELD_TYPE_LABEL,
        'visibility' => 'container',
      ),
      'workFullAddress' => array(
        'label' => 'general.header.fullAddress',
        'type' => self::FIELD_TYPE_TEXTAREA,
        'mapsTo' => 'visitFullAddress',
        'visibility_linked' => 'workAddressLabel',
        'rule' => 'max_chars:255'
      ),
      'workCountry' => array(
        'label' => 'general.message.field.country',
        'type' => self::FIELD_TYPE_SYSTEM,
        'mapsTo' => 'VisitCountryNode',
        'system_table' => 'Country',
        'visibility_linked' => 'workAddressLabel',
        'affects' => 'workRegion',
      ),
      'workRegion' => array(
        'label' => 'general.message.field.region',
        'type' => self::FIELD_TYPE_SYSTEM,
        'mapsTo' => 'VisitProvinceNode',
        'system_table' => 'Region',
        'visibility_linked' => 'workAddressLabel',
        'affectable' => true,
      ), // end work subsection
      'alternativeAddressLabel' => array(
        'label' => 'CxPortal.header.addressAlternative',
        'type' => self::FIELD_TYPE_LABEL,
        'visibility' => 'container',
      ),
      'alternativeFullAddress' => array(
        'label' => 'general.header.fullAddress',
        'type' => self::FIELD_TYPE_TEXTAREA,
        'mapsTo' => 'alternativeFullAddress',
        'visibility_linked' => 'homeAddressLabel',
        'rule' => 'max_chars:255'
      ),
      'alternativeCountry' => array(
        'label' => 'general.message.field.country',
        'type' => self::FIELD_TYPE_SYSTEM,
        'mapsTo' => 'AlternativeCountryNode',
        'system_table' => 'Country',
        'visibility_linked' => 'alternativeAddressLabel',
        'affects' => 'alternativeRegion',
      ),
      'alternativeRegion' => array(
        'label' => 'general.message.field.region',
        'type' => self::FIELD_TYPE_SYSTEM,
        'mapsTo' => 'AlternativeProvinceNode',
        'system_table' => 'Region',
        'visibility_linked' => 'alternativeAddressLabel',
        'affectable' => true,
      ),
    );
  }

  static function check_upload_feature_sanity() {
    $readonlies = Environment_Installation::isSane(true);
    if (is_array($readonlies)) {
      foreach ($readonlies as $readonly) {
        if (basename($readonly) == 'uploads') {
          return false;
        }
      }
    }

    return true;
  }

  static function display($settings, $publication_id = false, $preselected = array()) {
    if (!self::$DISPLAYED) {
      self::$DISPLAYED = true;
    } else {
      return;
    }

    self::$currentConfig = $settings;
    if (Environment_Settings::getFormAddressFormatGeneral() == 'international') {
      self::$fields['address']['items'] = self::addressInternationalFormat();
    }


    foreach (self::$fields as $group => $group_settings) {
      foreach ($group_settings['items'] as $item => $item_settings) {
        $parent = '';
        if (isset($item_settings['affected_by'])) {
          $parent = $item_settings['affected_by'];
          if (isset($settings[$parent . '_visible'])) {
            $settings[$item . '_visible'] = $settings[$parent . '_visible'];
          }
          if (isset($settings[$parent . '_required'])) {
            $settings[$item . '_required'] = $settings[$parent . '_required'];
          }

          if (!isset($settings[$parent . '_visible'])) {
            @$settings[$parent . '_visible'] = $settings[$item . '_visible'];
          }
          if (!isset($settings[$parent . '_required'])) {
            @$settings[$parent . '_required'] = $settings[$item . '_required'];
          }
        }
      }
    }

    $backtrace = debug_backtrace();
    foreach ($backtrace as $bkt) {
      if (isset($bkt['file'])) {
        if (basename($bkt['file']) == 'class-frontend.php')
          return '';
      }
    }
    $interfacer = WP_Interfacer::getInstance();
    $labels = Utils_Labels::get($interfacer->language_major_current());
    $uploadIsSane = self::check_upload_feature_sanity();
    $disableFields = false;
    if (!$uploadIsSane) {

      echo "<div class='carerix_error error'><p>{$labels['CxWpPlugin.message.uploadnotworking']}";
      if (isset($settings['cv_required']) || isset($settings['photo_required'])) {
        $disableFields = true;
        echo $labels['CxWpPlugin.message.reason'];
      }
      echo "</p></div>";
    }


    if (isset($_SESSION['employee_id'])) {
      $showParams = array('');
      foreach (self::$fields as $group => $_group_settings) {
        foreach (self::$fields[$group]['items'] as $item => $_settings) {
          switch (self::$fields[$group]['items'][$item]['type']) {
            case self::FIELD_TYPE_SYSTEM:
              if (isset(self::$fields[$group]['items'][$item]['mapsTo'])) {
                $showParams[] = 'to' . ucfirst(@self::$fields[$group]['items'][$item]['mapsTo']);
                $showParams[] = 'toUser.to' . ucfirst(@self::$fields[$group]['items'][$item]['mapsTo']);
              }
              break;
            default:
              if (isset(self::$fields[$group]['items'][$item]['mapsTo'])) {
                $showParams[] = @self::$fields[$group]['items'][$item]['mapsTo'];
                $showParams[] = 'toUser.' . @self::$fields[$group]['items'][$item]['mapsTo'];
              }
              break;
          }
        }
      }
      // [todo] build better showParams above
      //$params = array('show'=>array('', 'toUser', 'toExperienceNode', 'toUser.emailAddresses', 'emailAddressBusiness', 'emailAddressPrivate', 'emailAddress', ));
      $showParams = array_merge($showParams, array('toUser.emailAddresses',
        'emailAddressBusiness', 'emailAddressPrivate', 'emailAddress',
        'toUser.toGenderNode', 'toGenderNode', 'workHistories',
        'educations', 'skills', 'languages', 'attachments',
        'attachments.toTypeNode', 'attachments.toTypeNode.parentNodes',
        'toUser.toDoUsers', 'toUser', 'initials', '',
      ));
      $params = array('show' => $showParams);

      $candidateEmployeeObject = Carerix_Api_Rest_Entity_CREmployee::find((int) @$_SESSION['employee_id'], $params);
      $candidateUserObject = $candidateEmployeeObject->getToUser();

      $hasNewsletter = false;

      $newsletters = $candidateEmployeeObject->getToUser()->getId();
      if ($newsletters !== null)
        $hasNewsletter = true;

      self::$HAS_NEWSLETTER = $hasNewsletter;

      $uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::UPLOADS_DIR_CONTAINER;

      $can_not_upload = false;
      if (!is_dir($uploadDirPath)) {
        if (!mkdir($uploadDirPath, 0777, true)) {
          $can_not_upload = true;
        }
      }

      $attachments = $candidateEmployeeObject->getAttachments();

      if ($attachments !== null) {
        /** @var $lastCV Carerix_Api_Rest_Entity_CRAttachment */
        $lastCV = null;

        /** @var $lastPhoto Carerix_Api_Rest_Entity_CRAttachment */
        $lastPhoto = null;

        /** @var $extraDocuments Carerix_Api_Rest_Entity_CRAttachment[] */
        $extraDocuments = array();

        foreach ($attachments as $attachment) {
          // if document type has no class set, skip;
          if (!@$attachment->toTypeNode->parentNodes)
            continue;
          // if document type is set to 'not for web', skip;
          if (@$attachment->toTypeNode->isHidden == '1')
            continue;

          $currentAttachmentTime = strtotime($attachment->modificationDate);

          // decide the type of the attachment;
          foreach ($attachment->toTypeNode->parentNodes as $parentNode) {
            switch ($parentNode->value) {
              case 'CVTag':
                $attachmentType = 'cv';
                break;
              case 'PhotoTag':
                $attachmentType = 'photo';
                break;
              case 'DocumentUploadedByCandidateTag':
                $attachmentType = 'extra';
                break;
            }
          }

          // based on type of the attachment, apply the sorting
          switch ($attachmentType) {
            case 'cv':
              if ($lastCV === null) {
                $lastCV = $attachment;
              } else {
                $lastCVTime = strtotime($lastCV->modificationDate);
                if ($lastCVTime <= $currentAttachmentTime) {
                  $lastCV = $attachment;
                }
              }
              break;
            case 'photo':
              if ($lastPhoto === null) {
                $lastPhoto = $attachment;
              } else {
                $lastPhotoTime = strtotime($lastPhoto->modificationDate);
                if ($lastPhotoTime <= $currentAttachmentTime) {
                  $lastPhoto = $attachment;
                }
              }
              break;
            case 'extra':
              /* if has no PhotoTag and CVTag and reached this part, 
               *  it is considered as an extra document
               */
              array_push($extraDocuments, $attachment);
              break;
          }
        }

        if ($lastCV !== null) {
          self::_keep_attachment('cv', $lastCV, $candidateEmployeeObject, $uploadDirPath);
        }

        if ($lastPhoto) {
          self::_keep_attachment('photo', $lastPhoto, $candidateEmployeeObject, $uploadDirPath);
        }

        usort($extraDocuments, array('Entity_Form_EmployeeApply', 'sort_by_creation_time'));
        $counter = 1;
        foreach ($extraDocuments as $extraDocument) {
          if ($counter > 6)
            break;
          /** @var $extraDocument Carerix_Api_Rest_Entity_CRAttachment * */
          // for the extra documents, should be kept only those attachments that have DocumentUploadedByCandidateTag
          self::_keep_attachment('coverLetter', $extraDocument, $candidateEmployeeObject, $uploadDirPath, true);

          $counter++;
        }
      }

      self::$employee = $candidateEmployeeObject;
      self::$user = $candidateUserObject;

      self::$emails = array();
      foreach (self::$user->getEmailAddresses() as $address) {
        array_push(self::$emails, array('email' => $address->emailAddress, 'type' => $address->toEmailAddressLabel->dataNodeID));
      }
    }

    $input_data = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);
    $errors = array();


    if ($input_data !== NULL && isset($input_data['apply_action']) && wp_verify_nonce($input_data['_carerix_nonce_field'], 'apply_action')) {
      if (isset($input_data['linkedinurl_parse'])) {
        if (isset($input_data['linkedinurl']) && strlen(trim($input_data['linkedinurl'])) > 0) {
          $data = Environment_Parser::parse_linkedin_url($input_data['linkedinurl']);

          self::$employee = Carerix_Api_Rest_Entity_CREmployee::fromXml($data);
          self::$user = self::$employee->getToUser();

          $attachs = self::$employee->getAttachments();

          $uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::UPLOADS_DIR_CONTAINER;

          $can_not_upload = false;
          if (!is_dir($uploadDirPath)) {
            if (!mkdir($uploadDirPath, 0777, true)) {
              $can_not_upload = true;
            }
          }

          foreach ($attachs as $attach) {
            $name = $attach->filePath;
            $content = $attach->toAttachmentData->content;

            if (stripos($name, '_CV'))
              $type = 'cv';
            if (stripos($name, '_Photourl'))
              $type = 'photo';

            $extra = '_linkedin_' . $type . '_' . uniqid();
            $content = base64_decode($content);
            $name = $name . $extra;
            $path = $uploadDirPath . DIRECTORY_SEPARATOR . $name;
            if (!file_exists($path)) {
              file_put_contents($path, $content);
            }
            if (file_exists($path)) {
              $_SESSION['kept'][$type . '_on_disk'] = $name;
              $_SESSION['kept'][$type . '_to_display'] = $attach->filePath;
              $_SESSION['sent'][$type . '_on_disk'] = true;
            }
          }
        }
      } elseif (isset($input_data['cvupload_parse'])) {
        $file_real_path = $_FILES['cvupload']['tmp_name'];
        $name = $filename = $_FILES['cvupload']['name'];
        $filetype = $_FILES['cvupload']['type'];
        $acceptedTypes = array('application/octet-stream', 'application/msword', 'application/pdf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'text/plain');

        $uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::UPLOADS_DIR_CONTAINER;

        $can_not_upload = false;
        if (!is_dir($uploadDirPath)) {
          if (!mkdir($uploadDirPath, 0777, true)) {
            $can_not_upload = true;
          }
        }

        if (!$can_not_upload) {
          $extra = '_cv_parsed_' . date('Y_m_d_H_i_s');
          $destination_path = $uploadDirPath . DIRECTORY_SEPARATOR . $name . $extra;

          move_uploaded_file($file_real_path, $destination_path);
          $file_real_path = $destination_path;
          $_SESSION['kept']['cv_on_disk'] = $name . $extra;
          $_SESSION['kept']['cv_to_display'] = $name;
        }
        if (is_file($file_real_path) && in_array($filetype, $acceptedTypes)) {

          $data = utf8_encode(Environment_Parser::parse_cv_upload($file_real_path, $filename));
          self::$employee = Carerix_Api_Rest_Entity_CREmployee::fromXml($data);

          self::$user = self::$employee->getToUser();
        }
      } else {
        $errors = self::process($input_data, $settings, $publication_id);
        if (count($errors) == 0) {
            
          self::localizeConfirmationPage($settings, $interfacer);
        
          $data = array(
            'back' => $interfacer->get_post_permalink(),
            'settings' => $settings
          );

          $template = new Utils_Template('fragments/frontend/apply_successful');
          $template->with($data)->display();
          return;
        } else {
          if (isset($_SESSION['sent'])) {
            foreach ($_SESSION['sent'] as $name => $sent) {
              if (stripos($name, '_online_') !== FALSE) {
                $_SESSION['sent'][$name] = false;
              }
            }
          }
        }
      }
    }


    echo '<div id="apply_form">';
    if (isset($errors) && isset($errors['general'])) {
      echo '<div class="error">' . $errors['general'] . '</div>';
    }

    echo '<form autocomplete="off" method="post" enctype="multipart/form-data">';
    echo '<input style="display:none" type="text" name="fakeusernameremembered"/>';
    echo '<input style="display:none" type="password" name="fakepasswordremembered"/>';
    echo '<input type="hidden" name="apply_action" value="1" />';
    wp_nonce_field('apply_action', '_carerix_nonce_field');
    

    if (!isset($_SESSION['logged_in']) || !$_SESSION['logged_in']) {
      $apply_status = Environment_Settings::getFormApplyOptionsStatus();
      if ($apply_status == 'on') {
        $link = $interfacer->get_post_permalink(false, 'apply_mode=true');
        $label = $labels['CxPortal.message.text.logon'];
        echo '<div><a href="' . $link . '">' . $label . '</a></div>';
      }
    }

    $lastLabel = '';
    $fieldsAvailable = self::$fields;
    if (isset($settings['field_order'])) {
      $field_order = json_decode(html_entity_decode($settings['field_order']));
      if ($field_order) {
        $_collector = array();
        foreach ($field_order as $section => $fields) {
          if (!isset($_collector[$section]))
            $_collector[$section] = array();

          if ($section == 'address' && Environment_Settings::getFormAddressFormatGeneral() == 'international') {
            $addressFields = self::addressInternationalFormat();
            $fields = array_keys($addressFields);
          }

          $_collector[$section] = $fieldsAvailable[$section];
          $_collector[$section]['items'] = array();


          foreach ($fields as $field) {
            if (!isset($fieldsAvailable[$section]['items'][$field]))
              continue;
            $_collector[$section]['items'][$field] = $fieldsAvailable[$section]['items'][$field];

            while (@$fieldsAvailable[$section]['items'][$field]['affects']) {
              $field = $fieldsAvailable[$section]['items'][$field]['affects'];
              $_collector[$section]['items'][$field] = $fieldsAvailable[$section]['items'][$field];
            }
            
            $relatedFields = @$fieldsAvailable[$section]['items'][$field]['related'];
            if (is_array($relatedFields)) {
                foreach($relatedFields as $field_name) {
                    $_collector[$section]['items'][$field_name] = $fieldsAvailable[$section]['items'][$field_name];
                }
            }
          }
        }
        $fieldsAvailable = $_collector;
      }
    }

    foreach ($fieldsAvailable as $group_name => $group_settings) {
      self::displaySection($group_name, $group_settings, $settings, $errors);
    }

    $settings['rememberme_visible'] = true;
    if (isset($settings['rememberme_visible'])) {
      echo self::_remember_me($disableFields);
    }

    $settings['required_visible'] = true;
    if (isset($settings['required_visible'])) {
      echo self::_agree_html($disableFields);
    }

    if (Environment_Settings::getApplyFormCaptchaStatus() == 'on') {
      $hasError = false;
      if (isset($errors['captcha']))
        $hasError = true;
      self::_display_captcha($hasError);
    }

    $apply_label = $labels['general.button.apply'];
    self::_print_age_limit();

    if (!$disableFields)
      echo '<input id="apply_button" type="submit" value="' . $apply_label . '" /></form></div>';
  }

  static function _display_captcha($error = false) {
    @session_start();
    include(CARERIX_INSTALL_DIR . DIRECTORY_SEPARATOR . 'captcha' . DIRECTORY_SEPARATOR . 'simple-php-captcha.php');
    $_SESSION['captcha'] = simple_php_captcha();
    $labels = Utils_Labels::get();

    echo "
            <div>
              <label for='captcha'>{$labels['general.message.text.captchaText']}</label>
              <input type='text' name='captcha_code' value='' autocomplete='off' /><br />
              <img src='{$_SESSION['captcha']['image_src']}' alt='CAPTCHA code'>
        ";

    if ($error) {
      echo '<span class="error">' . $labels['CxWpPlugin.message.captcha'] . '</span>';
    }
    echo "</div>";
  }

  static function _remember_me($disabled = false) {
    $labels = Utils_Labels::get();
    $rememberMeLabel = $labels['general.message.field.rememberMe'];
    $checked = '';
    $disabled = $disabled ? 'disabled' : '';

    if (isset($_POST['_remember_me']) || isset($_SESSION['logged_in'])) {
      $checked = 'checked';
    }
    $agreeText = "<div id='remember_me'><label><input name='_remember_me' type='checkbox' $disabled value='on' $checked />&nbsp; $rememberMeLabel </label></div>";
    return $agreeText;
  }
  
  static function _agree_html($disabled = false) {
    $agreementText = Environment_Settings::getFormGeneralAgreement();
    $agreementLabel = Environment_Settings::getFormGeneralAgreementLabel();
    $agreementUrl = Environment_Settings::getFormGeneralAgreementUrl();
    $checked = '';
    $disabled = $disabled ? 'disabled' : '';
    $agreeDiv = '';
    $agreeText = '';
    $labels = Utils_Labels::get();
    if (isset($_POST['_agreement_check'])) {
      $checked = 'checked';
    }

    if ($agreementLabel) {
      if ($agreementUrl) {
        $agreeText = "<div id='agreement_container'><label for='agreement_check'><input $disabled id='agreement_check' name='_agreement_check' type='checkbox' value='on' value='on' $checked />&nbsp;<a href='$agreementUrl' target='_blank'>$agreementLabel</a> </label></div>";
      } else if ($agreementText) {
        $agreeDiv = "<div id='agreement' style='display: none;'><p><button class='close_modal_carerix'>{$labels['general.button.close']}</button></p><p class='text'>$agreementText</p><p><button class='close_modal_carerix'>{$labels['general.button.close']}</button></p></div>";
        $agreeText = "<div id='agreement_container'><label for='agreement_check'><input $disabled id='agreement_check' name='_agreement_check' type='checkbox' value='on' value='on' $checked />&nbsp;<a href='#' id='show_agreement'>$agreementLabel</a> </label></div>";
      }
    }

    return $agreeDiv . $agreeText;
  }

  static function _repeat_html($id) {
    $showRemove = false;
    switch ($id) {
      case 'languages':
        if (self::$profLanguagesGroupCount > 0) {
          $index = self::$profLanguagesGroupCount - 1;
          $id = "{$id}_index_{$index}";
          $showRemove = true;
        }
        break;
      case 'professionalSkills':
        if (self::$profSkillsGroupCount > 0) {
          $index = self::$profSkillsGroupCount - 1;
          $id = "{$id}_index_{$index}";
          $showRemove = true;
        }
        break;
      case 'education':
        if (self::$educationGroupCount > 0) {
          $index = self::$educationGroupCount - 1;
          $id = "{$id}_index_{$index}";
          $showRemove = true;
        }

        break;
      case 'workExperience':
        if (self::$workGroupCount > 0) {
          $index = self::$workGroupCount - 1;
          $id = "{$id}_index_{$index}";
          $showRemove = true;
        }

        break;
    }

    if ($showRemove) {
      $removeStyle = '';
    } else {
      $removeStyle = 'style="display: none;"';
    }

    $labels = Utils_Labels::get();

    return '<div><a href="#" data-repeats="' . $id . '" class="block-repeat ' . $id . '"><i class="fa fa-plus"></i> ' . $labels['general.button.add'] . '</a>' .
        '<a href="#" data-remove="' . $id . '" class="block-derepeat ' . $id . '" ' . $removeStyle . '><i class="fa fa-minus"></i> ' . $labels['general.button.remove'] . '</a></div>';
  }

  static function _print_age_limit() {
    $ageLimit = (int) Environment_Settings::getAgeLimit();
    $labels = Utils_Labels::get();
    $message = $labels['CxWpPlugin.message.dateofbirth'];

    $message = str_replace('{0}', $ageLimit, $message);

    echo "<script type='text/javascript'>window.carerix_age_limit = $ageLimit; window.carerix_age_limit_message = '$message';</script>";
  }

  static function process($input_data, $settings, $publication_id) {
    $errors = array();
    $labels = Utils_Labels::get();
    $requiredMessage = $labels['CxFinder.message.error.emptySearchSite'];
    $validFields = self::$fields;
    foreach ($input_data as $input_group_name => $input_group_fields) {
      if (isset($validFields[$input_group_name])) {
        foreach ($input_group_fields as $field => $value) {
          if (is_scalar($value)) {
            if (isset($settings[$field . '_required'])) {
              if (strlen(trim($value)) <= 0) {
                $errors[$field] = $requiredMessage;
              }
            }

            if (isset($validFields[$input_group_name]['items'][$field]['rule'])) {
              $rules = $validFields[$input_group_name]['items'][$field]['rule'];

              $_rtable = explode('|', $rules);
              foreach ($_rtable as $rulestring) {
                list($rule, $rule_argument) = explode(':', $rulestring);
                $rule_result = Entity_Form_Rules::$rule($rule_argument, $value);
                if ($rule_result !== TRUE) {
                  $errors[$field] = $rule_result;
                }
              }
            }
          }
          if (is_array($value)) {
            if (isset($settings[$field . '_required'])) {
              foreach ($value as $index => $_v) {
                if (strlen(trim($_v)) <= 0) {
                  $errors[$field] = $requiredMessage;
                  break;
                }
              }
            }
          }
        }
      }
    }


    if (isset($_FILES['personal']['name']) && is_array($_FILES['personal']['name'])) {
      foreach ($_FILES['personal']['name'] as $key => $value) {
        if ($key == 'coverLetter')
          continue;
        if (!$value)
          continue;
        $acceptedType = array();
        switch ($key) {
          case 'cv':
            $acceptedType = array('doc', 'pdf', 'docx');
            break;
          case 'photo':
            $acceptedType = array('png', 'jpg', 'jpeg', 'gif');
            break;
        }
        if (is_array($value)) {
          $value = $value[0];
        }
        if ($value) {
          $filebasename = $value;

          $filetype = '';
          $parts = explode('.', $filebasename);
          $filetype = strtolower($parts[count($parts) - 1]);

          if (!in_array($filetype, $acceptedType)) {
            $errors[$key] = 'Invalid file type';
          }
        }
      }
    }

    $captchaStatus = Environment_Settings::getApplyFormCaptchaStatus();
    $labels = Utils_Labels::get();
    if ($captchaStatus == 'on' && strtolower($input_data['captcha_code']) !== strtolower($_SESSION['captcha']['code'])) {
      $errors['captcha'] = $labels['CxWpPlugin.message.captcha'];
    }

    /**
     * @fixme: should become unrequired only in certain cases.
     */
    $toUnsets = array('desiredFunction1_function1', 'desiredFunction2_function1', 'desiredFunction3_function1', 'position_level_2', 'education_1', 'education_2', 'alternativeRegion', 'homeRegion', 'desiredRegion_1', 'desiredRegion_2', 'desiredRegion_3');
    foreach ($toUnsets as $toUnset) {
      if (isset($errors[$toUnset]))
        unset($errors[$toUnset]);
    }

    // all required fields were provided
    // build the employee object

    if (count($errors) == 0) {
      //$employee = self::buildEmployee($input_data, $publication_id);

      try {
        $employee = self::highDefinitionBuildEmployee($input_data);
        $params = array('x-cx-pub' => $publication_id);
        $answerEmployee = $employee->apply($params);
        Utils_Log::write(Utils_Log::TYPE_APPLY_SEND, $employee);

        if (isset($input_data['motivationAndSource']['motivation'])) {
          $publication = Carerix_Api_Rest_Entity_CRPublication::find($publication_id, array('show' => 'toVacancy.vacancyID'));
          $vacancy_id = $publication->getToVacancy()->getId();
          $employee_id = $answerEmployee->getId();

          $match_id = "{$vacancy_id}.{$employee_id}";
          try {
            $match = Carerix_Api_Rest_Entity_CRMatch::find($match_id);
            $match->setMotivation(trim($input_data['motivationAndSource']['motivation']));
            $match->save();
          } catch (Carerix_Api_Rest_Exception $care) {
            /* $request = Carerix_Api_Rest_Entity::getLastRequest();
              $response = Carerix_Api_Rest_Entity::getLastResponse(); */
          }
        }
        
        $employee = Carerix_Api_Rest_Entity_CREmployee::find($answerEmployee->getId(), array('show' => array('toUser.userID', 'toUser.toDoUsers', 'toUser.emailAddresses','hobbies')));
        
        if (isset($input_data['personal']['hobbies'])) {
            $hobbies = $employee->getHobbies();
            
            if ($hobbies) {
                $hobbies .= "\n\n[" . date('Y-m-d H:i:s') . "]\n" . $input_data['personal']['hobbies'];
            } else {
                $hobbies = $input_data['personal']['hobbies'];
            }
            
            $employee->setHobbies($hobbies);
            $employee->save();
        }
        
        $emails = $employee->getToUser()->emailAddresses;
        $toRemove = array();
        $processed = array();
        foreach ($emails as $email) {
          $emailAddressNormalized = trim(strtolower($email->emailAddress));

          if (isset($processed[$emailAddressNormalized])) {
            $toRemove[] = $email;
          }

          if (!in_array($emailAddressNormalized, $_POST['contact']['email'])) {
            $toRemove[] = $email;
          }

          $processed[$emailAddressNormalized] = true;
        }

        foreach ($toRemove as $emailToDelete) {
          $emailToDelete->delete();
        }

        if (isset($input_data['newsletterSubscription']['subscribe'])) {
          // find the first newsletter;
          $newsletters = Carerix_Api_Rest_Entity_CRNewsletter::findAll(array(
                'show' => array('isConfidential', 'deleted', 'name'),
                'qualifier' => 'isConfidential = 0'
          ));


          if (isset($newsletters[0])) {
            $newsletter = $newsletters[0];

            $user_id = $employee->getToUser()->getId();
            $newsletter_id = $newsletter->getId();
            $todoUserId = "{$newsletter_id}.{$user_id}";

            $toDoUsers = $employee->getToUser()->getToDoUsers();
            $isAssigned = false;
            if ($toDoUsers) {
              foreach ($toDoUsers as $_todouser) {
                if ($_todouser->getId() == $todoUserId) {
                  $isAssigned = true;
                  break;
                }
              }
            }
            if (!$isAssigned) {
              $user = new Carerix_Api_Rest_Entity_CRUser();
              $user->setId($employee->getToUser()->getId());

              $todo = new Carerix_Api_Rest_Entity_CRToDo();
              $todo->setId($newsletter->getId());

              $toDoUser = new Carerix_Api_Rest_Entity_CRToDoUser();
              $toDoUser->setToToDo($todo);
              $toDoUser->setIsActive(1);
              $user->setToDoUsers(array($toDoUser));
              $user->save();
              if ($user)
                Utils_Log::write('NewsletterSubscribe', $user);
            }
          }
        }

        if (isset($_POST['_remember_me'])) {
          $_SESSION['logged_in'] = true;
          $_SESSION['name'] = @$answerEmployee->firstName
              . ' ' . @$answerEmployee->lastNamePrefix
              . ' ' . @$answerEmployee->lastName;
          $_SESSION['employee_id'] = @$answerEmployee->employeeID;
        }

        //unset($_SESSION['kept']);
      } catch (Exception $ex) {
        Utils_Log::write(Utils_Log::TYPE_APPLY_SEND, $employee, $ex);
        $errors['general'] = 'Unfortunately, we can not process your request at this time';
      }
    }

    if (isset($_SESSION['sent'])) {
      foreach ($_SESSION['sent'] as $name => $sent) {
        if (stripos($name, '_online_') === FALSE) {
          $_SESSION['sent'][$name] = false;
        }
      }
    }
    if (count($errors) > 0) {
      self::get_attachments();
      unset($_SESSION['sent']);
    }

    return $errors;
  }

  static function getUploadedFile($fname, $type = '', $fi_name = false, $tmpname = false, $repeat = false) {
    if (!$type)
      $type = self::DOCUMENT_CV_ATTACHMENT;
    $_field_name = $fname;
    $_fname = $fi_name ? $fi_name : @$_FILES['personal']['name'][$fname];
    $tmpname = $tmpname ? $tmpname : @$_FILES['personal']['tmp_name'][$fname];
    $fname = $_fname;

    $uploadDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . Utils_Log::UPLOADS_DIR_CONTAINER;
    $input_data = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);
    $can_not_upload = false;

    if (!is_dir($uploadDirPath)) {
      if (!mkdir($uploadDirPath, 0777, true)) {
        $can_not_upload = true;
      }
    }

    if (is_string($tmpname) && file_exists($tmpname)) {
      $name = $fname;
      $path = $tmpname;

      if (!$can_not_upload) {
        $extra = '_' . $type . '_' . date('Y_m_d_H_i_s');
        $destination_path = $uploadDirPath . DIRECTORY_SEPARATOR . $name . $extra;
        if ($repeat) {
          $_SESSION['kept'][$_field_name . '_on_disk'][] = $name . $extra;
          $_SESSION['kept'][$_field_name . '_to_display'][] = $name;
          $_SESSION['sent'][$_field_name . '_on_disk'][] = $name . $extra;
        } else {
          $_SESSION['kept'][$_field_name . '_on_disk'] = $name . $extra;
          $_SESSION['kept'][$_field_name . '_to_display'] = $name;
          $_SESSION['sent'][$_field_name . '_on_disk'] = $name . $extra;
        }

        move_uploaded_file($path, $destination_path);

        // get type from system
        //$params = array('show'=>array('dataNodeID'), 'qualifier'=> "value = '$type'");
        if (!file_exists($destination_path))
          return false;
        $base64encoded = base64_encode(file_get_contents($destination_path));
        file_put_contents($destination_path, $base64encoded);

        $params = array('show' => array('dataNodeID', 'parentNodes.value', 'value'), 'qualifier' => "parentNodes.value = '$type'");
        $typeNodes = Carerix_Api_Rest_Entity_CRDataNode::findAll($params);
        //$typeNode = $typeNodes[0];
        // create attachment
        $attach = new Carerix_Api_Rest_Entity_CRAttachment();
        // get the first datanode that has this type tag as parent
        $typeNodeGathered = null;
        foreach ($typeNodes as $typeNode) {

          if ($type == self::DOCUMENT_EXTRA_TAG) {
            $shouldUse = true;
            foreach ($typeNode->parentNodes as $parentNode) {
              if ($parentNode->value == self::DOCUMENT_CV_ATTACHMENT || $parentNode->value == self::DOCUMENT_PHOTO_TAG) {
                $shouldUse = false;
                break;
              }
            }
            if ($shouldUse) {
              $typeNodeGathered = $typeNode;
            }
          } else {
            $typeNodeGathered = $typeNode;
          }

          break;
        }

        if ($typeNodeGathered) {
          $attach->setToTypeNode($typeNodeGathered);
        }

        $attach->setFilePath($name);


        $attach->setContent($base64encoded);

        return $attach;
      }
    } else {
      if (isset($input_data["{$_field_name}_on_disk"]) || isset($_SESSION['kept'][$_field_name . '_on_disk'])) {
        if (isset($_SESSION['sent'][$_field_name . '_on_disk']) && $_SESSION['sent'][$_field_name . '_on_disk'] == true
        ) {
          return false;
        }
        $name = isset($input_data["{$_field_name}_on_disk"]) ? $input_data["{$_field_name}_on_disk"] : $_SESSION['kept'][$_field_name . '_on_disk'];

        $display_name = isset($input_data["{$_field_name}_to_display"]) ? $input_data["{$_field_name}_to_display"] : $_SESSION['kept'][$_field_name . '_to_display'];
        if (is_array($name))
          return false; // @todo REMOVE!
        $destination_path = $uploadDirPath . DIRECTORY_SEPARATOR . $name;
        if (!file_exists($destination_path))
          return false;
        $params = array('show' => array('dataNodeID', 'parentNodes.value'), 'qualifier' => "parentNodes.value = '$type'");
        $typeNodes = Carerix_Api_Rest_Entity_CRDataNode::findAll($params);

        // create attachment
        $attach = new Carerix_Api_Rest_Entity_CRAttachment();
        // get the first datanode that has this type tag as parent
        foreach ($typeNodes as $typeNode) {
          $attach->setToTypeNode($typeNode);
          break;
        }

        $attach->setFilePath($display_name);

        $attach->setContent(file_get_contents($destination_path));
        $_SESSION['sent'][$_field_name . '_on_disk'] = true;
        return $attach;
      }
    }
    return false;
  }

  static function _check($field, $input_data) {
    $value_to_return = false;
    if (isset($input_data[$field]) && is_string($input_data[$field]) && strlen(trim($input_data[$field])) > 0) {
      $value_to_return = $input_data[$field];
    } else if (isset($input_data[$field]) && is_array($input_data[$field]) && count($input_data[$field]) > 0) {
      $value_to_return = $input_data[$field];
    }

    if (is_string($value_to_return)) {
      $value_to_return = trim($value_to_return);
    }

    return $value_to_return;
  }

  // the discoverability of CxRest API is lacking, 
  // so it's best to assign the values manually
  static function highDefinitionBuildEmployee($input_data) {
    $employee = new Carerix_Api_Rest_Entity_CREmployee();
    $user = new Carerix_Api_Rest_Entity_CRUser();
    $userSet = false;
    // processing personal section

    $section_data = @$input_data['personal'];
    if (FALSE !== $value = self::_check('lastName', $section_data))
      $employee->setLastName($value);
    if (FALSE !== $value = self::_check('middleName', $section_data))
      $employee->setLastNamePrefix($value);
    if (FALSE !== $value = self::_check('lastNameAtBirth', $section_data))
      $employee->setBirthName($value);
    if (FALSE !== $value = self::_check('initials', $section_data)) {
      $user->setInitials($value);
      $userSet = true;
    }
    if (FALSE !== $value = self::_check('firstName', $section_data))
      $employee->setFirstName($value);
    if (FALSE !== $value = self::_check('title', $section_data))
      $employee->setTitle($value);
    if (FALSE !== $value = self::_check('suffix', $section_data))
      $employee->setSuffix($value);
    if (FALSE !== $value = self::_check('password', $section_data)) {
      if ($value != '******')
        $user->setPassword($value);
    }

    if (FALSE !== $value = self::_check('gender', $section_data)) {
      $genderID = 0;
      if (is_array($value)) {
        $genderID = $value[0];
      } else {
        $genderID = $value;
      }
      $genderNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $genderID);
      $user->setToGenderNode($genderNode);
      $userSet = true;
    }
    if (FALSE !== $value = self::_check('maritalStatus', $section_data)) {
      $maritalStatusNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $value);
      $employee->setToMaritalStatusNode($maritalStatusNode);
    }
    if (FALSE !== $value = self::_check('birthDate', $section_data)) {
      $value = date('Y-m-d 00:00:00', strtotime($value));
      $user->setBirthDate($value);
      $employee->setBirthDate($value);
      $userSet = true;
    }
    if (FALSE !== $value = self::_check('birthPlace', $section_data))
      $employee->setBirthCity($value);
    if (FALSE !== $value = self::_check('countryOfBirth', $section_data)) {
      $birthCountryNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $value);
      $employee->setToBirthCountryNode($birthCountryNode);
    }
    if (FALSE !== $value = self::_check('nationality', $section_data)) {
      $nationalityNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $value);
      $employee->setToNationalityNode($nationalityNode);
    }
    if (FALSE !== $value = self::_check('functionLevel', $section_data)) {
      $functionLevelNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $value);
      $employee->setToWorkLevelNode($functionLevelNode);
    }
    if (FALSE !== $value = self::_check('experience', $section_data)) {
      $experienceNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $value);
      $employee->setToExperienceNode($experienceNode);
    }

    // contact details
    $section_data = @$input_data['contact'];

    if (FALSE !== $emails = self::_check('email', $section_data)) {
      $emailsSet = array();
      $emailTypes = self::_check('email', @$input_data['type-contact']);
      for ($i = 0; $i < count($emails); $i++) {
        $email = new Carerix_Api_Rest_Entity_CREmailAddress();
        $email->setEmailAddress($emails[$i]);
        if ((int) $emailTypes[$i][$i]) {
          $type = Carerix_Api_Rest_Entity_CRDataNode::find((int) $emailTypes[$i][$i]);
          $email->setToEmailAddressLabel($type);
        }
        array_push($emailsSet, $email);
      }

      if (count($emailsSet) > 0) {
        $user->setEmailAddresses($emailsSet);
      }
      $userSet = true;
    }
    if (FALSE !== $homePhone = self::_check('homePhone', $section_data)) {
      $employee->setPhoneNumber($homePhone);
    }
    if (FALSE !== $workPhone = self::_check('workPhone', $section_data)) {
      $employee->setPhoneNumberBusiness($workPhone);
    }
    if (FALSE !== $mobileNumber = self::_check('mobileNumber', $section_data)) {
      $employee->setMobileNumber($mobileNumber);
    }
    if (FALSE !== $mobileNumberBusiness = self::_check('mobileNumberBusiness', $section_data)) {
      $employee->setMobileNumberBusiness($mobileNumberBusiness);
    }
    if (FALSE !== $contactInstructions = self::_check('contactInstructions', $section_data)) {
      $employee->setContactInfo($contactInstructions);
    }

    // address section
    if (isset($input_data['address'])) {
      
    }
    $section_data = @$input_data['address'];

    // home address
    if (FALSE !== $homeStreetName = self::_check('homeStreetName', $section_data)) {
      $user->setHomeStreet($homeStreetName);
    }
    if (FALSE !== $homeStreetNumber = self::_check('homeStreetNumber', $section_data)) {
      $user->setHomeNumber($homeStreetNumber);
    }
    if (FALSE !== $homeSuffixNumber = self::_check('homeSuffixNumber', $section_data)) {
      $user->setHomeNumberSuffix($homeSuffixNumber);
    }
    if (FALSE !== $homeCity = self::_check('homeCity', $section_data)) {
      $user->setHomeCity($homeCity);
    }
    if (FALSE !== $homePostalCode = self::_check('homePostalCode', $section_data)) {
      $user->setHomePostalCode($homePostalCode);
    }
    if (FALSE !== $homeCountry = self::_check('homeCountry', $section_data)) {
      $homeCountry = Carerix_Api_Rest_Entity_CRDataNode::find((int) $homeCountry);
      $user->setToHomeCountryNode($homeCountry);
    }
    if (FALSE !== $homeRegion = self::_check('homeRegion', $section_data)) {
      $homeRegion = Carerix_Api_Rest_Entity_CRDataNode::find((int) $homeRegion);
      $user->setToHomeProvinceNode($homeRegion);
    }

    // [VP] To trigger update of home address we need to set empty value for homeFullAddress
    if ($homeRegion || $homeCountry || $homeCity || $homeSuffixNumber || $homeStreetNumber || $homeStreetName) {
        $user->setHomeFullAddress('');
    }

    // work address
    if (FALSE !== $workStreetName = self::_check('workStreetName', $section_data)) {
      $user->setVisitStreet($workStreetName);
      $userSet = true;
    }
    if (FALSE !== $workStreetNumber = self::_check('workStreetNumber', $section_data)) {
      $user->setVisitNumber($workStreetNumber);
      $userSet = true;
    }
    if (FALSE !== $workSuffixNumber = self::_check('workSuffixNumber', $section_data)) {
      $user->setVisitNumberSuffix($workSuffixNumber);
      $userSet = true;
    }
    if (FALSE !== $workCity = self::_check('workCity', $section_data)) {
      $user->setVisitCity($workCity);
      $userSet = true;
    }
    if (FALSE !== $workPostalCode = self::_check('workPostalCode', $section_data)) {
      $user->setVisitPostalCode($workPostalCode);
      $userSet = true;
    }
    if (FALSE !== $visitCountry = self::_check('workCountry', $section_data)) {
      $visitCountry = Carerix_Api_Rest_Entity_CRDataNode::find((int) $visitCountry);
      $user->setToVisitCountryNode($visitCountry);
      $userSet = true;
    }
    if (FALSE !== $visitRegion = self::_check('workRegion', $section_data)) {
      $visitRegion = Carerix_Api_Rest_Entity_CRDataNode::find((int) $visitRegion);
      $user->setToVisitProvinceNode($visitRegion);
      $userSet = true;
    }
    
    // alternative address

    if (FALSE !== $alternativeStreetName = self::_check('alternativeStreetName', $section_data)) {
      $user->setAlternativeStreet($alternativeStreetName);
      $userSet = true;
    }
    if (FALSE !== $alternativeStreetNumber = self::_check('alternativeStreetNumber', $section_data)) {
      $user->setAlternativeNumber($alternativeStreetNumber);
      $userSet = true;
    }
    if (FALSE !== $alternativeSuffixNumber = self::_check('alternativeSuffixNumber', $section_data)) {
      $user->setAlternativeNumberSuffix($alternativeSuffixNumber);
      $userSet = true;
    }
    if (FALSE !== $alternativeCity = self::_check('alternativeCity', $section_data)) {
      $user->setAlternativeCity($alternativeCity);
      $userSet = true;
    }
    if (FALSE !== $alternativePostalCode = self::_check('alternativePostalCode', $section_data)) {
      $user->setAlternativePostalCode($alternativePostalCode);
      $userSet = true;
    }
    if (FALSE !== $alternativeCountry = self::_check('alternativeCountry', $section_data)) {
      $alternativeCountry = Carerix_Api_Rest_Entity_CRDataNode::find((int) $alternativeCountry);
      $user->setToAlternativeCountryNode($alternativeCountry);
      $userSet = true;
    }
    if (FALSE !== $alternativeRegion = self::_check('alternativeRegion', $section_data)) {
      $alternativeRegion = Carerix_Api_Rest_Entity_CRDataNode::find((int) $alternativeRegion);
      $user->setToAlternativeProvinceNode($alternativeRegion);
      $userSet = true;
    }
    if (FALSE !== $homeFullAddress = self::_check('homeFullAddress', $section_data)) {
      $employee->setFullAddress($homeFullAddress);
      $userSet = true;
    }
    if (FALSE !== $workFullAddress = self::_check('workFullAddress', $section_data)) {
      $user->setVisitFullAddress($workFullAddress);
      $userSet = true;
    }
    if (FALSE !== $alternativeFullAddress = self::_check('alternativeFullAddress', $section_data)) {
      $user->setAlternativeFullAddress($alternativeFullAddress);
      $userSet = true;
    }

    // section wishes
    $section_data = @$input_data['wishes'];
    if (FALSE !== $employment = self::_check('employment', $section_data)) {
      $employment = Carerix_Api_Rest_Entity_CRDataNode::find((int) $employment);
      $employee->setToPrefContractTypeNode($employment);
      $employee->setToContractTypeNode($employment);
    }
    if (FALSE !== $hoursPerWeek = self::_check('hoursPerWeek', $section_data)) {
      $employee->setMaxFte($hoursPerWeek);
    }
    if (FALSE !== $minSalary = self::_check('minSalary', $section_data)) {
      $employee->setMinSalary($minSalary);
    }
    if (FALSE !== $minSalaryCurrencyNode = self::_check('minSalaryCurrencyNode', $section_data)) {
      $minSalaryCurrencyNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $minSalaryCurrencyNode);
      $employee->setToMinSalaryCurrencyNode($minSalaryCurrencyNode);
    }
    if (FALSE !== $minSalaryUnitNode = self::_check('minSalaryUnitNode', $section_data)) {
      $minSalaryUnitNode = Carerix_Api_Rest_Entity_CRDataNode::find((int) $minSalaryUnitNode);
      $employee->setToMinSalaryUnitNode($minSalaryUnitNode);
    }
    if (FALSE !== $desiredFunction1 = self::_check('desiredFunction1', $section_data)) {
      $desiredFunction1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction1);
      //   $employee->setToPrefBranche1Level1Node($desiredFunction1);
      $employee->setToProductTypeNode1($desiredFunction1);
    }
    if (FALSE !== $desiredFunction1_function0 = self::_check('desiredFunction1_function0', $section_data)) {
      $desiredFunction1_function0 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction1_function0);
      $employee->setToLevel1Function1($desiredFunction1_function0);
    }
    if (FALSE !== $desiredFunction1_function1 = self::_check('desiredFunction1_function1', $section_data)) {
      $desiredFunction1_function1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction1_function1);
      $employee->setToLevel2Function1($desiredFunction1_function1);
    }

    if (FALSE !== $desiredFunction2 = self::_check('desiredFunction2', $section_data)) {
      $desiredFunction2 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction2);
      // $employee->setToPrefBranche2Level1Node($desiredFunction2);
      $employee->setToProductTypeNode2($desiredFunction2);
    }
    if (FALSE !== $desiredFunction2_function0 = self::_check('desiredFunction2_function0', $section_data)) {
      $desiredFunction2_function0 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction2_function0);
      $employee->setToLevel1Function2($desiredFunction2_function0);
    }
    if (FALSE !== $desiredFunction2_function1 = self::_check('desiredFunction2_function1', $section_data)) {
      $desiredFunction2_function1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction2_function1);
      $employee->setToLevel2Function2($desiredFunction2_function1);
    }

    if (FALSE !== $desiredFunction3 = self::_check('desiredFunction3', $section_data)) {
      $desiredFunction3 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction3);
      //$employee->setToPrefBranche3Level1Node($desiredFunction3);
      $employee->setToProductTypeNode3($desiredFunction3);
    }
    if (FALSE !== $desiredFunction3_function0 = self::_check('desiredFunction3_function0', $section_data)) {
      $desiredFunction3_function0 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction3_function0);
      $employee->setToLevel1Function3($desiredFunction3_function0);
    }
    if (FALSE !== $desiredFunction3_function1 = self::_check('desiredFunction3_function1', $section_data)) {
      $desiredFunction3_function1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredFunction3_function1);
      $employee->setToLevel2Function3($desiredFunction3_function1);
    }

    if (FALSE !== $desiredCountry_1 = self::_check('desiredCountry_1', $section_data)) {
      $desiredCountry_1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredCountry_1);
      $employee->setToPrefCountry1Node($desiredCountry_1);
    }
    if (FALSE !== $desiredRegion_1 = self::_check('desiredRegion_1', $section_data)) {
      $desiredRegion_1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredRegion_1);
      $employee->setToPrefRegion1Node($desiredRegion_1);
    }

    if (FALSE !== $desiredCountry_2 = self::_check('desiredCountry_2', $section_data)) {
      $desiredCountry_2 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredCountry_2);
      $employee->setToPrefCountry2Node($desiredCountry_2);
    }
    if (FALSE !== $desiredRegion_2 = self::_check('desiredRegion_2', $section_data)) {
      $desiredRegion_2 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredRegion_2);
      $employee->setToPrefRegion2Node($desiredRegion_2);
    }

    if (FALSE !== $desiredCountry_3 = self::_check('desiredCountry_3', $section_data)) {
      $desiredCountry_3 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredCountry_3);
      $employee->setToPrefCountry3Node($desiredCountry_3);
    }
    if (FALSE !== $desiredRegion_3 = self::_check('desiredRegion_3', $section_data)) {
      $desiredRegion_3 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $desiredRegion_3);
      $employee->setToPrefRegion3Node($desiredRegion_3);
    }
    
    if (FALSE !== $prefBranche1Level1Node = self::_check('prefBranche1Level1Node', $section_data)) {
      $prefBranche1Level1Node = Carerix_Api_Rest_Entity_CRDataNode::find((int) $prefBranche1Level1Node);
      $employee->setToPrefBranche1Level1Node($prefBranche1Level1Node);
    }
    if (FALSE !== $prefBranche1Level2Node = self::_check('prefBranche1Level2Node', $section_data)) {
      $prefBranche1Level2Node = Carerix_Api_Rest_Entity_CRDataNode::find((int) $prefBranche1Level2Node);
      $employee->setToPrefBranche1Level2Node($prefBranche1Level2Node);
    }
    
    if (FALSE !== $prefBranche2Level1Node = self::_check('prefBranche2Level1Node', $section_data)) {
      $prefBranche2Level1Node = Carerix_Api_Rest_Entity_CRDataNode::find((int) $prefBranche2Level1Node);
      $employee->setToPrefBranche2Level1Node($prefBranche2Level1Node);
    }
    if (FALSE !== $prefBranche2Level2Node = self::_check('prefBranche2Level2Node', $section_data)) {
      $prefBranche2Level2Node = Carerix_Api_Rest_Entity_CRDataNode::find((int) $prefBranche2Level2Node);
      $employee->setToPrefBranche2Level2Node($prefBranche2Level2Node);
    }
    
    if (FALSE !== $prefBranche3Level1Node = self::_check('prefBranche3Level1Node', $section_data)) {
      $prefBranche3Level1Node = Carerix_Api_Rest_Entity_CRDataNode::find((int) $prefBranche3Level1Node);
      $employee->setToPrefBranche3Level1Node($prefBranche3Level1Node);
    }
    if (FALSE !== $prefBranche3Level2Node = self::_check('prefBranche3Level2Node', $section_data)) {
      $prefBranche3Level2Node = Carerix_Api_Rest_Entity_CRDataNode::find((int) $prefBranche3Level2Node);
      $employee->setToPrefBranche3Level2Node($prefBranche3Level2Node);
    }
    

    // ambition & availability
    $section_data = @$input_data['ambitionAndAvailability'];

    if (FALSE !== $ambition = self::_check('ambition', $section_data)) {
      $employee->setAmbition($ambition);
    }
    if (FALSE !== $availableIn = self::_check('availableIn', $section_data)) {
      $availableIn = Carerix_Api_Rest_Entity_CRDataNode::find((int) $availableIn);
      $employee->setToAvailableWithinNode($availableIn);
    }
    if (FALSE !== $availableFrom = self::_check('availableFrom', $section_data)) {
      $availableFrom = date('Y-m-d 00:00:00', strtotime($availableFrom));
      $employee->setAvailableFromDate($availableFrom);
    }

    // transportation section
    $section_data = @$input_data['transportation'];
    if (FALSE !== $homeWork = self::_check('homeWork', $section_data)) {
      $homeWork = Carerix_Api_Rest_Entity_CRDataNode::find((int) $homeWork);
      $employee->setToMeansOfTransportNode($homeWork);
    }
    if (FALSE !== $haveCar = self::_check('haveCar', $section_data)) {
      if (in_array(strtolower($haveCar), array('on', 1, '1'))) {
        $haveCar = 1;
      } else {
        $haveCar = 0;
      }
      $employee->setHasCar((int) $haveCar);
    }

    if (FALSE !== $license = self::_check('license', $section_data)) {
      $license = Carerix_Api_Rest_Entity_CRDataNode::find((int) $license);
      $employee->setToDriverLicenceNode($license);
    }
    if (FALSE !== $issuingPlace = self::_check('issuingPlace', $section_data)) {
      $employee->setDriversLicenseCity($issuingPlace);
    }
    if (FALSE !== $licenceNo = self::_check('licenceNo', $section_data)) {
      $employee->setDriversLicenseCode($licenceNo);
    }
    if (FALSE !== $numberValidity = self::_check('numberValidity', $section_data)) {
      $numberValidity = date('Y-m-d 00:00:00', strtotime($numberValidity));
      $employee->setDriversLicenseValidUntilDate($numberValidity);
    }


    // work experience section
    $section_data = @$input_data['workExperience'];

    $workHistories = array();
    for ($i = 0; $i < count($section_data['employer']); $i++) {
      $employer = @$section_data['employer'][$i];
      $periodOf = date('Y-m-d 00:00:00', strtotime(@$section_data['periodOf'][$i]));
      $periodUntil = date('Y-m-d 00:00:00', strtotime(@$section_data['periodUntil'][$i]));
      $functionName = @$section_data['functionName'][$i];
      $workLocation = @$section_data['workLocation'][$i];
      $position = @$section_data['position'][$i];
      $position_level_1 = @$section_data['position_level_1'][$i];
      $position_level_2 = @$section_data['position_level_2'][$i];
      $industry = @$section_data['industry'][$i];
      $sector = @$section_data['sector'][$i];
      $assignment = @$section_data['assignment'][$i];

      $workHistory = new Carerix_Api_Rest_Entity_CRWorkHistory();
      if ($employer)
        $workHistory->setEmployer($employer);
      if ($periodOf)
        $workHistory->setStartDate($periodOf);
      if ($periodUntil)
        $workHistory->setEndDate($periodUntil);
      if ($functionName)
        $workHistory->setFunction($functionName);
      if ($workLocation)
        $workHistory->setWorkLocation($workLocation);

      if ($position) {
        $position = Carerix_Api_Rest_Entity_CRDataNode::find((int) $position);
        $workHistory->setToProductTypeNode($position);
      }

      if ($position_level_1) {
        $position_level_1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $position_level_1);
        $workHistory->setToFunctionLevel1($position_level_1);
      }

      if ($position_level_2) {
        $position_level_2 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $position_level_2);
        $workHistory->setToFunctionLevel2($position_level_2);
      }

      if ($industry) {
        $industry = Carerix_Api_Rest_Entity_CRDataNode::find((int) $industry);
        $workHistory->setToBrancheLevel1($industry);
      }

      if ($sector) {
        $sector = Carerix_Api_Rest_Entity_CRDataNode::find((int) $sector);
        $workHistory->setToBrancheLevel2($sector);
      }

      if ($assignment) {
        $workHistory->setInfo($assignment);
      }

      if ($employer || $periodOf || $periodUntil || $functionName || $functionName || $workLocation || $position || $position_level_1 || $position_level_2 || $industry || $sector || $assignment) {
        $workHistories[] = $workHistory;
      }
    }
    if (count($workHistories) > 0)
      $employee->setWorkHistories($workHistories);

    $section_data = @$input_data['experienceOther'];

    if (FALSE !== $experienceOther = self::_check('experiencenotes', $section_data)) {
      $employee->setExperienceInformation($experienceOther);
    }

    // education
    $section_data = @$input_data['education'];
    $educations = array();

    for ($i = 0; $i < count($section_data['education']); $i++) {
      $education = @$section_data['education'][$i];
      $education_1 = @$section_data['education_1'][$i];
      $education_2 = @$section_data['education_2'][$i];
      $institute = @$section_data['institute'][$i];
      $yearStart = @$section_data['yearStart'][$i];
      $yearEnd = @$section_data['yearEnd'][$i];

      $educationObject = new Carerix_Api_Rest_Entity_CREmployeeEducation();

      if ($education) {
        $education = Carerix_Api_Rest_Entity_CRDataNode::find((int) $education);
        $educationObject->setToLevel1Education1($education);
      }
      if ($education_1) {
        $education_1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $education_1);
        $educationObject->setToLevel1Education2($education_1);
      }
      if ($education_2) {
        $education_2 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $education_2);
        $educationObject->setToLevel1Education3($education_2);
      }
      if ($institute)
        $educationObject->setInstitute($institute);
      if ($yearStart) {
        $yearStart = "{$yearStart}-00-00 00:00:00";
        $educationObject->setStartDate($yearStart);
      }
      if ($yearEnd) {
        $yearEnd = "{$yearEnd}-00-00 00:00:00";
        $educationObject->setEndDate($yearEnd);
      }

      if ($education || $education_1 || $education_2 || $institute || $yearStart || $yearEnd) {
        $educations[] = $educationObject;
      }
    }
    if (count($educations) > 0)
      $employee->setEducations($educations);

    $section_data = @$input_data['educationOther'];
    if (FALSE !== $educationOther = self::_check('languagenotes', $section_data)) {
      $employee->setEducationInformation($educationOther);
    }

    // skills
    $section_data = @$input_data['professionalSkills'];
    $skills = array();
    for ($i = 0; $i < count($section_data['skill']); $i++) {
      $skill = @$section_data['skill'][$i];
      $skill_1 = @$section_data['skill_1'][$i];
      $skill_2 = @$section_data['skill_2'][$i];

      $skillObject = new Carerix_Api_Rest_Entity_CREmployeeSkill();

      if ($skill) {
        $skill = Carerix_Api_Rest_Entity_CRDataNode::find((int) $skill);
        $skillObject->setToSkillLevel1($skill);
      }

      if ($skill_1) {
        $skill_1 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $skill_1);
        $skillObject->setToSkillLevel2($skill_1);
      }

      if ($skill_2) {
        $skill_2 = Carerix_Api_Rest_Entity_CRDataNode::find((int) $skill_2);
        $skillObject->setToSkillLevel3($skill_2);
      }

      if ($skill || $skill_1 || $skill_2) {
        $skills[] = $skillObject;
      }
    }
    if (count($skills) > 0)
      $employee->setSkills($skills);

    $section_data = @$input_data['skillsOther'];

    if (FALSE !== $skillsOther = self::_check('skillnotes', $section_data)) {
      $employee->setSkillNotes($skillsOther);
    }

    // skills
    $section_data = @$input_data['languages'];
    $languages = array();
    for ($i = 0; $i < count($section_data['language']); $i++) {
      if (!$section_data['language'][$i])
        continue;

      $language = $section_data['language'][$i];
      $read = $section_data['read'][$i];
      $write = $section_data['write'][$i];
      $speak = $section_data['speak'][$i];

      $language = Carerix_Api_Rest_Entity_CRDataNode::find((int) $language);

      $languageObject = new Carerix_Api_Rest_Entity_CREmployeeLanguage();
      $languageObject->setToLanguage($language);

      if ($read) {
        $read = Carerix_Api_Rest_Entity_CRDataNode::find((int) $read);
        $languageObject->setToLanguageReadLevel($read);
      }

      if ($write) {
        $write = Carerix_Api_Rest_Entity_CRDataNode::find((int) $write);
        $languageObject->setToLanguageWriteLevel($write);
      }

      if ($speak) {
        $speak = Carerix_Api_Rest_Entity_CRDataNode::find((int) $speak);
        $languageObject->setToLanguageSpeakLevel($speak);
      }

      $languages[] = $languageObject;
    }
    if (count($languages) > 0)
      $employee->setLanguages($languages);

    $section_data = @$input_data['languagesOther'];
    if (FALSE !== $languagesOther = self::_check('languagenotes', $section_data)) {
      $employee->setLanguageNotes($languagesOther);
    }

    $section_data = @$input_data['motivationAndSource'];

    if (FALSE !== $source = self::_check('source', $section_data)) {
      $source = Carerix_Api_Rest_Entity_CRDataNode::find((int) $source);
      $employee->setToSourceNode($source);
    }
    if (FALSE !== $sourceOther = self::_check('sourceOther', $section_data)) {
      $employee->setSourceInfo($sourceOther);
    }
    
    if ($userSet) {
      $employee->setToUser($user);
    }

    $attachments = self::get_attachments();

    if (count($attachments) > 0) {
      $employee->setAttachments($attachments);
    }
    return $employee;
  }

  static function get_attachments() {
    $attachments = array();
    // get the cv
    $cvAttach = self::getUploadedFile('cv', self::DOCUMENT_CV_ATTACHMENT);

    if ($cvAttach !== false) {
      $attachments[] = $cvAttach;
    }

    // get the photo
    $photoAttach = self::getUploadedFile('photo', self::DOCUMENT_PHOTO_TAG);
    if ($photoAttach !== false) {
      $attachments[] = $photoAttach;
    }

    if (isset($_FILES['personal']['name']['coverLetter']) && is_array($_FILES['personal']['name']['coverLetter'])) {
      for ($i = 0; $i < count($_FILES['personal']['name']['coverLetter']); $i++) {
        $fname = $_FILES['personal']['name']['coverLetter'][$i];
        $tmpname = $_FILES['personal']['tmp_name']['coverLetter'][$i];
        $extraDocumentAttach = self::getUploadedFile('coverLetter', self::DOCUMENT_EXTRA_TAG, $fname, $tmpname, true);
        if ($extraDocumentAttach !== false) {
          $attachments[] = $extraDocumentAttach;
        }
      }
    } else {
      $extraDocumentAttach = self::getUploadedFile('coverLetter', self::DOCUMENT_EXTRA_TAG, true);
      if ($extraDocumentAttach !== false) {
        $attachments[] = $extraDocumentAttach;
      }
    }

    return $attachments;
  }

  static function display_input_field($group, $field_name, $is_required = false, $error = false, $forced = false, $value = false, $disabled = false, $child = false, $recurse = false) {
    $keep_value = $value;

    $field_type = self::$fields[$group]['items'][$field_name]['type'];

    $mapper = 'placeholder';
    if (isset(self::$fields[$group]['items'][$field_name]['mapsTo'])) {
      $mapper = self::$fields[$group]['items'][$field_name]['mapsTo'];
    }

    $section_type = self::$fields[$group]['type'];

    $subsection_type = false;
    if (isset(self::$fields[$group]['subtype'])) {
      $subsection_type = self::$fields[$group]['subtype'];
    }

    if ($section_type != 'individual' && $subsection_type != 'individual' && !$forced) {
      return;
    }

    $input_data = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);
    $input_data = apply_filters('carerix_apply_form', $input_data);
    if ($value === false)
      $value = '';
    if ($input_data != null && !$value) {
      if (!$recurse)
        $extractValue = @$input_data[$group][$field_name];
      else
        $extractValue = $value;
      if (is_array($extractValue)) {
        $continueFlow = false;
        // @todo: conditions for the rest of repeatable groups
        // in bezit van auto is not reselected, also.
        switch ($group) {
          case 'contact':
            foreach ($extractValue as $_value) {
              if (is_scalar($_value) && !empty($_value)) {
                self::display_input_field($group, $field_name, $is_required, $error, $forced, $_value, $disabled, $child);
              }
            }
            break;
          case 'workExperience':
            $fields = array_keys($input_data[$group]);
            $totalCount = count($input_data[$group][$fields[0]]);

            if ($totalCount <= self::$workGroupCount) {
              break;
            }
            $index = self::$workGroupCount;
            echo "<div id='{$group}_index_{$index}' data-index='$index'>";
            foreach ($fields as $field_name) {
              $_value = $input_data[$group][$field_name][self::$workGroupCount];
              self::display_input_field($group, $field_name, $is_required, $error, $forced, $_value, $disabled, $child, true);
            }
            echo "</div>";
            self::$workGroupCount += 1;
            break;
          case 'education':
            $fields = array_keys($input_data[$group]);
            $totalCount = count($input_data[$group][$fields[0]]);
            //echo "</div>";
            if ($totalCount <= self::$educationGroupCount) {
              break;
            }
            $index = self::$educationGroupCount;
            echo "<div id='{$group}_index_{$index}' data-index='$index'>";
            foreach ($fields as $field_name) {
              $_value = $input_data[$group][$field_name][self::$educationGroupCount];
              self::display_input_field($group, $field_name, $is_required, $error, $forced, $_value, $disabled, $child, true);
            }
            echo "</div>";
            self::$educationGroupCount += 1;
            break;
          case 'professionalSkills':
            $fields = array_keys($input_data[$group]);
            $totalCount = count($input_data[$group][$fields[0]]);
            //echo "</div>";
            if ($totalCount <= self::$profSkillsGroupCount) {
              break;
            }
            $index = self::$profSkillsGroupCount;

            echo "<div id='{$group}_index_{$index}' data-index='$index'>";
            foreach ($fields as $field_name) {
              $_value = $input_data[$group][$field_name][self::$profSkillsGroupCount];
              self::display_input_field($group, $field_name, $is_required, $error, $forced, $_value, $disabled, $child, true);
            }
            echo "</div>";

            self::$profSkillsGroupCount += 1;
            break;
          case 'languages':
            $fields = array_keys($input_data[$group]);
            $totalCount = count($input_data[$group][$fields[0]]);
            //echo "</div>";
            if ($totalCount <= self::$profLanguagesGroupCount) {
              break;
            }
            $index = self::$profLanguagesGroupCount;
            echo "<div id='{$group}_index_{$index}' data-index='$index'>";
            foreach ($fields as $field_name) {
              $_value = $input_data[$group][$field_name][self::$profLanguagesGroupCount];
              self::display_input_field($group, $field_name, $is_required, $error, $forced, $_value, $disabled, $child, true);
            }
            echo "</div>";
            self::$profLanguagesGroupCount += 1;
            break;
          default:
            $continueFlow = true;
            break;
        }

        if (!$continueFlow) {
          return;
        }
        $value = @$extractValue[0];
        //return;
      } else {
        $value = $extractValue;
      }
    }

    if (self::$employee != null && !$value) {
      $candidateEmployeeObject = self::$employee;
      $candidateUserObject = self::$user;


      if ($mapper == 'EmailAddress') {
        $mapper = 'emailAddressBusiness';
        $field_type = 'tempo';
      }

      switch ($field_type) {
        case self::FIELD_TYPE_PASSWORD:
        case self::FIELD_TYPE_FILE:
        case self::FIELD_TYPE_GROUP:
        case self::FIELD_TYPE_PASSWORD:
          break;
        case self::FIELD_TYPE_SYSTEM:

          $method = 'getTo' . ucfirst($mapper);
          try {
            $_node = $candidateEmployeeObject->$method();
            if ($_node)
              $value = $_node->getDataNodeID();
          } catch (Exception $ex) {
            try {
              $_node = $candidateUserObject->$method();
              if ($_node)
                $value = $_node->getDataNodeID();
            } catch (Exception $ex) {
              
            }
          }
          break;
        default:
          $method = 'get' . ucfirst($mapper);

          $tryAgain = false;
          try {
            $value = $candidateEmployeeObject->$method();
            if (is_string($value)) {
              if (strlen(trim($value)) == 0) {
                $tryAgain = true;
              }
            }

            if (is_array($value)) {
              if (count($value) == 0) {
                $tryAgain = true;
              }
            }
          } catch (Exception $ex) {
            try {
              $value = $candidateUserObject->$method();
            } catch (Exception $ex) {
              
            }
          }
          if ($tryAgain) {
            try {
              $value = $candidateUserObject->$method();
            } catch (Exception $ex) {
              
            }
          }

          break;
      }
    }
    if (is_array($value) && count($value) == 0)
      $value = '';

    $fragment_path = 'fragments/input/text';
    $interfacer = WP_Interfacer::getInstance();
    $labels = Utils_Labels::get($interfacer->language_major_current());

    $class_chain = 'carerix_input';
    if ($is_required) {
      $class_chain .= ' carerix_required';
      $class_chain .= " $group-$field_name";
    }




    $data = array(
      'label' => @$labels[self::$fields[$group]['items'][$field_name]['label']],
      'name' => "{$group}[{$field_name}]",
      'id' => "$group-$field_name",
      'value' => $value,
      'error' => $error,
      'class_chain' => $class_chain,
      'repeatable' => isset(self::$fields[$group]['items'][$field_name]['repeatable']) ? true : false,
      'disabled' => $disabled,
      'child' => $child
    );

    if ($data['repeatable'] || self::$fields[$group]['type'] == 'group') {
      $data['name'] = $data['name'] . '[]';
    }

    if (isset(self::$fields[$group]['items'][$field_name]['systemtype'])) {
      $data['prefixoptions'] = self::get_system_select_options(self::$fields[$group]['items'][$field_name]['systemtype']);

      $data['prefixid'] = "$group-$field_name";
    }

    switch ($field_type) {
      case self::FIELD_TYPE_PASSWORD:
        $fragment_path = 'fragments/input/password';
        break;
      case self::FIELD_TYPE_SYSTEM:

        $deffield = strtolower(self::$fields[$group]['items'][$field_name]['system_table']);
        if (!$value && isset(self::$currentConfig['default_' . $deffield])) {
          $value = self::$currentConfig['default_' . $deffield];
          $data['value'] = $value;
        }

        $data['options'] = self::get_system_select_options(self::$fields[$group]['items'][$field_name]['system_table']);

        if (isset(self::$fields[$group]['items'][$field_name]['affects'])) {
          $data['affects'] = $group . '-' . self::$fields[$group]['items'][$field_name]['affects'];
        }
        if (isset(self::$fields[$group]['items'][$field_name]['affectable'])) {
          $data['affectable'] = true;
        }
        if (isset(self::$fields[$group]['items'][$field_name]['affected_by'])) {
          $data['affected_by'] = $group . '-' . self::$fields[$group]['items'][$field_name]['affected_by'];
        }
        $data['table'] = self::$fields[$group]['items'][$field_name]['system_table'];

        if ($field_name == 'gender') {
          $fragment_path = 'fragments/input/system_radio';
        } else {
          $fragment_path = 'fragments/input/system_select';
        }

        break;
      case self::FIELD_TYPE_FILE:
        // get the value from here
        $fragment_path = 'fragments/input/file';
        break;
      case self::FIELD_TYPE_CHECK:
        $fragment_path = 'fragments/input/check';
        if (isset($input_data[$group][$field_name]) && ($input_data[$group][$field_name] == 1 || $input_data[$group][$field_name] == 'on' )) {
          $data['checked'] = 'checked';
        }

        break;
      case self::FIELD_TYPE_DATE:
        $fragment_path = 'fragments/input/date';
        break;
      case self::FIELD_TYPE_LABEL:
        $fragment_path = 'fragments/labels/subsection';
        break;
      case self::FIELD_TYPE_TEXTAREA:
        $fragment_path = 'fragments/input/textarea';
        break;
      case self::FIELD_TYPE_TEXT:
      default:
        $fragment_path = 'fragments/input/text';
        break;
    }
    if ($data['id'] == 'newsletterSubscription-subscribe') {
      if (self::$HAS_NEWSLETTER)
        $data['checked'] = 'checked';
    }
    if ($group == 'professionalSkills') {
      if (!$data['value']) {
        $data['value'] = $keep_value;
      }
    }
    $input_text_fragment = new Utils_Template($fragment_path);
    $input_text_fragment->with($data)->display();
  }

  static function get_system_select_options($table, $parent = false) {
    $interfacer = WP_Interfacer::getInstance();
    $language = $interfacer->get_carerix_language();
    
    $cache_session_key = $table;
    if ($parent) {
        $cache_session_key .= '_parent_' . $parent;
    }
    if ($language) {
        $cache_session_key .= '_lang_' . $language;
    }
    $cache_session_key = md5($cache_session_key);
    
    if (FALSE !== $cached_data = Utils_Cache::get($cache_session_key)) {
      return $cached_data;
    }
    
    $labels = Utils_Labels::get($interfacer->language_major_current());

    $options = array();
    $params = array('type' => $table, 'show' =>array('value', 'dataNodeID', 'isHidden', 'parentNodes.value'), 'qualifier' => " isHidden = 0 " , 'language' => $language);
    if ($parent) {
      $params['parent'] = $parent;
    }
    $table_values = array();
    $exception = false;
    try {

      $table_values = Carerix_Api_Rest_Entity_CRDataNode::listBy($params);
    } catch (Carerix_Api_Rest_Exception $care) {
      $exception = $care;
    } catch (Exception $ex) {
      $exception = $ex;
    }
    if ($exception) {
      Utils_Log::write(Utils_Log::TYPE_DATANODE, false, $exception);
    }


    foreach ($table_values as $table_item) {
      if ($table_item->isHidden)
        continue;

      $label = trim($table_item->value);

      // special case, could fail if something changes in Carerix 
      if ($table == 'E-mailadres-label') {
        switch ($label) {
          case 'E-mail adres prive':
            $label = $labels['general.message.field.emailPrivateSimple'];

            break;
          case 'E-mail adres zakelijk':
            $label = $labels['general.message.field.emailWorkSimple'];
            break;
        }
      }
      if (Environment_Settings::getFormGenderUnknown() !== 'on') {
        if ($table == 'Salutation') {
          $toContinue = false;
          foreach ($table_item->parentNodes as $pn) {
            if ($pn->value == 'UnknownTag')
              $toContinue = true;
          }
          if ($toContinue)
            continue;
        }
      }

      $option = array(
        'value' => $table_item->dataNodeID,
        'label' => $label
      );
      array_push($options, $option);
    }

    Utils_Cache::set($cache_session_key, $options);
    return $options;
  }

  static function get_parent($child) {
    $cache_session_key = md5('parent_for_' . $child);
    if (FALSE !== $cached_data = Utils_Cache::get($cache_session_key))
      return $cached_data;

    $params = array('show' => array('value', 'dataNodeID', 'isHidden', 'parentNodes'));

    $table_values = array();
    $exception = false;
    try {

      $table_values = Carerix_Api_Rest_Entity_CRDataNode::find($child, $params);
    } catch (Carerix_Api_Rest_Exception $care) {
      $exception = $care;
    } catch (Exception $ex) {
      $exception = $ex;
    }
    if ($exception) {
      Utils_Log::write(Utils_Log::TYPE_DATANODE, false, $exception);
    }
    if (count($table_values) < 1)
      return false;

    foreach ($table_values->parentNodes as $parent) {
      return $parent->dataNodeID;
    }
    return false;
  }

  /**
   * 
   * @param string $type
   * @param Carerix_Api_Rest_Entity_CRAttachment $attachment
   * @param Carerix_Api_Rest_Entity_CREmployee $employee
   */
  private static function _keep_attachment($type, $attachment, $employee, $directory, $repeatable = false) {
    $name_suffix = '_online_b64' . $type . $employee->getId();
    if (!$attachment->content)
      return;
    $content = $attachment->content;
    // because sometimes carerix sends characters in a path that are not valid
    $sanitized_file_name = sanitize_file_name($attachment->filePath);
    $file_base_name = $sanitized_file_name . $name_suffix;
    $file_path = $directory . DIRECTORY_SEPARATOR . $file_base_name;

    if (!file_exists($file_path)) {
      file_put_contents($file_path, $content);
    }
    clearstatcache();
    if (file_exists($file_path)) {
      if ($repeatable) {
        if (isset($_SESSION['kept'][$type . '_on_disk']) && is_array($_SESSION['kept'][$type . '_on_disk']) && in_array($file_base_name, $_SESSION['kept'][$type . '_on_disk'])) {
          
        } else {
          $_SESSION['kept'][$type . '_on_disk'][] = $file_base_name;
          $_SESSION['kept'][$type . '_to_display'][] = $sanitized_file_name;
          $_SESSION['sent'][$type . '_on_disk'][] = true;
        }
      } else {
        $_SESSION['kept'][$type . '_on_disk'] = $file_base_name;
        $_SESSION['kept'][$type . '_to_display'] = $sanitized_file_name;
        $_SESSION['sent'][$type . '_on_disk'] = true;
      }
    }
  }

  /**
   * biggest date first (newest)
   * @param Carerix_Api_Rest_Entity_CRAttachment $att1
   * @param Carerix_Api_Rest_Entity_CRAttachment $att2
   */
  public static function sort_by_modification_time($att1, $att2) {
    if ($att1->modificationDate == $att2->modificationDate)
      return 0;
    return ($att1->modificationDate > $att2->modificationDate) ? -1 : +1;
  }

  /**
   * biggest date first (newest)
   * @param Carerix_Api_Rest_Entity_CRAttachment $att1
   * @param Carerix_Api_Rest_Entity_CRAttachment $att2
   */
  public static function sort_by_creation_time($att1, $att2) {
    if ($att1->creationDate == $att2->creationDate)
      return 0;
    return ($att1->creationDate > $att2->creationDate) ? -1 : +1;
  }

  public static function displaySection($group_name, $group_settings, $form_settings, $errors) {

    $displayFunction = 'displaySection_' . $group_name;
    if (is_callable(array('Entity_Form_EmployeeApply', $displayFunction))) {
      $input = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);
      $hasVisibleFields = false;
      $visibleFields = array();
      $requiredFields = array();
      foreach ($group_settings['items'] as $item_name => &$item_settings) {
        $affectedBy = @$group_settings['items'][$item_name]['affected_by'];
        if (isset($form_settings[$affectedBy . '_visible'])) {
          $form_settings[$item_name . '_visible'] = $form_settings[$affectedBy . '_visible'];
        }

        if (isset($form_settings[$affectedBy . '_required'])) {
          $form_settings[$item_name . '_required'] = $form_settings[$affectedBy . '_required'];
        }

        if (isset($form_settings[$item_name . '_visible']) || isset($form_settings[$item_name . '_required'])) {
          $hasVisibleFields = true;

          array_push($visibleFields, $item_name);
          if (isset($form_settings[$item_name . '_required'])) {
            $requiredFields[$item_name] = true;
          }
        }
      }

      if ($hasVisibleFields) {
        $group_values = array();
        if (isset($input[$group_name])) {
          $group_values = $input[$group_name];
        }
        self::$displayFunction($group_settings, $visibleFields, $requiredFields, $errors, $group_values);
      }
    } else {
      
    }
  }

  private static function displaySection_personal($group_settings, $visibleFields, $requiredFields, $errors, $values) {

    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'personal';

    foreach ($visibleFields as $field) {
      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;
      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => $labels[$group_settings['items'][$field]['label']],
        'name' => "personal[{$field}]",
        'id' => "personal-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain,
      );
      switch ($field) {
        case 'gender':
          if (is_array($value)) {
            $val = '';
            foreach ($value as &$_v) {
              $val = $_v;
              break;
            }
            $data['value'] = $val;
          }

          if (!$data['value'] && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);
            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
            $data['value'] = $value;
          }

          $table = $group_settings['items'][$field]['system_table'];
          $options = self::get_system_select_options($table);
          $data['options'] = $options;
          $fragment = 'applyform/system_radio';
          break;
        case 'cv':
        case 'photo':
          $fragment = 'applyform/file';
          break;
        case 'coverLetter':
          $fragment = 'applyform/repeatable_file';
          $data['repeatable'] = true;
          break;
        case 'password':
          $data['value'] = '******';
          $fragment = 'applyform/password';
          break;
        case 'maritalStatus':
        case 'countryOfBirth':
        case 'nationality':
        case 'functionLevel':

          $table = $group_settings['items'][$field]['system_table'];
          $options = self::get_system_select_options($table);
          $data['options'] = $options;
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);
            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/system_select';
          break;
        case 'experience':
        case 'birthDate':

          if (!$value && self::$employee) {
            $_m = $field;
            switch ($field) {
              case 'experience':
                $_m = 'experienceSinceDate';
                break;
              case 'birthDate':
                $_m = 'birthDate';
                break;
            }
            $method = 'get' . ucfirst($_m);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              $value = '';
            }
            $data['value'] = $value;
          }
          $fragment = 'applyform/date';
          break;
        
        case 'hobbies': 
            // Do not display existing value and on save append new data with "\n\n$timestamp"
            $fragment = 'applyform/textarea';
            break;
        default:
          if (!$value && self::$employee) {
            $_m = $field;
            switch ($field) {
              case 'middleName':
                $_m = 'lastNamePrefix';
                break;
              case 'lastNameAtBirth':
                $_m = 'birthName';
                break;
              case 'birthPlace':
                $_m = 'birthCity';
                break;
            }
            $method = 'get' . ucfirst($_m);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              $value = '';
            }
            $data['value'] = $value;
          }
          $fragment = 'applyform/text';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_contact($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $group = 'contact';
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";

    foreach ($visibleFields as $field) {
      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;
      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => $labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        case 'homePhone':
            if (!$value && self::$employee) {
              $value = self::$employee->getPhoneNumber();
            }
            $data['value'] = $value;
            $fragment = 'applyform/text';
            break;
        case 'workPhone':
            if (!$value && self::$employee) {
              $value = self::$user->getPhoneNumberBusiness();
            }
            $data['value'] = $value;
            $fragment = 'applyform/text';
            break;
        case 'mobileNumber':
            if (!$value && self::$user) {
              $value = self::$user->getMobileNumber();
            }
            $data['value'] = $value;
            $fragment = 'applyform/text';
            break;
        case 'mobileNumberBusiness':
            if (!$value && self::$user) {
              $value = self::$user->getMobileNumberBusiness();
            }
            $data['value'] = $value;
            $fragment = 'applyform/text';
            break;
        case 'email':
          $emails = array('');
          $emailTypes = array();
          if (isset($values['email'])) {
            $emails = $values['email'];
          } elseif (self::$employee) {

            $_emails = self::$user->getEmailAddresses();
            $emails = array();
            
            if ($_emails) {
                foreach ($_emails as $email) {
                    array_push($emails, $email->getEmailAddress());
                    array_push($emailTypes, $email->getToEmailAddressLabel()->getId());
                }
            }
          }
          $emailCount = count($emails);
          $data['index'] = uniqid();
          foreach ($emails as $index => $email) {
            $data['name'] = "{$group}[{$field}][]";
            $data['value'] = $email;
            $data['prefixoptions'] = self::get_system_select_options(self::$fields['contact']['items']['email']['systemtype']);

            $data['prefixid'] = "contact-{$index}";
            $data['count'] = $index;
            $data['emailTypes'] = $emailTypes;
            $fragment = 'applyform/text';
            if ($index >= 1) {
              $class_chain = str_replace('carerix_required', '', $class_chain);
              $data['class_chain'] = $class_chain;
            }
            if ($index >= $emailCount - 1) {
              $data['repeatable'] = true;
            }

            if ($fragment) {
              $input_text_fragment = new Utils_Template($fragment);
              $input_text_fragment->with($data)->display();
            }
          }
          break;
        case 'contactInstructions':
          if (!$value && self::$employee) {
            $value = self::$employee->getContactInfo();
          }
          $data['value'] = $value;
          $fragment = 'applyform/text';
          break;
      }

      if ($fragment && $field != 'email') {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_address($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'address';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => $labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        case 'homeAddressLabel':
        case 'workAddressLabel':
        case 'alternativeAddressLabel':
          $fragment = 'applyform/subsection';
          break;
        case 'homeStreetName':
        case 'workStreetName':
        case 'alternativeStreetName':

        case 'homeStreetNumber':
        case 'workStreetNumber':
        case 'alternativeStreetNumber':

        case 'homeSuffixNumber':
        case 'workSuffixNumber':
        case 'alternativeSuffixNumber':

        case 'homeCity':
        case 'workCity':
        case 'alternativeCity':

        case 'homePostalCode':
        case 'workPostalCode':
        case 'alternativePostalCode':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;

          $fragment = 'applyform/text';
          break;
        case 'homeCountry':
        case 'workCountry':
        case 'alternativeCountry':
        case 'homeRegion':
        case 'workRegion':
        case 'alternativeRegion':
          $table = $group_settings['items'][$field]['system_table'];
          $options = self::get_system_select_options($table);
          $data['options'] = $options;
          $data['table'] = $table;
          if (isset($group_settings['items'][$field]['affects'])) {
            $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
          }
          if (isset($group_settings['items'][$field]['affectable'])) {
            $data['affectable'] = true;
          }
          if (isset($group_settings['items'][$field]['affected_by'])) {
            $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
          }

          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);
            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;

          $fragment = 'applyform/system_select';

          break;
        case 'homeFullAddress':
        case 'workFullAddress':
        case 'alternativeFullAddress':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;

          $fragment = 'applyform/textarea';
          break;
          break;
        default:
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_wishes($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'wishes';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        case 'minSalary':
            $data['inlineblock'] = true;
            $data['container_class'] = 'cx_applyform_salary';
        case 'hoursPerWeek':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/text';
          break;
          
        case 'minSalaryUnitNode':
        case 'minSalaryCurrencyNode':
            $data['container_class'] = 'cx_applyform_salary';
        case 'employment':
        case 'desiredFunction1':
        case 'desiredFunction1_function0':
        case 'desiredFunction1_function1':
        case 'desiredFunction2':
        case 'desiredFunction2_function0':
        case 'desiredFunction2_function1':
        case 'desiredFunction3':
        case 'desiredFunction3_function0':
        case 'desiredFunction3_function1':
        case 'desiredCountry_1':
        case 'desiredRegion_1':
        case 'desiredCountry_2':
        case 'desiredRegion_2':
        case 'desiredCountry_3':
        case 'desiredRegion_3':
        case 'prefBranche1Level1Node':
        case 'prefBranche1Level2Node':
        case 'prefBranche2Level1Node':
        case 'prefBranche2Level2Node':
        case 'prefBranche3Level1Node':
        case 'prefBranche3Level2Node':
          $table = $group_settings['items'][$field]['system_table'];

          $options = self::get_system_select_options($table);
          $data['options'] = $options;
          $data['table'] = $table;
          if (isset($group_settings['items'][$field]['affects'])) {
            $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
          }
          if (isset($group_settings['items'][$field]['affectable'])) {
            $data['affectable'] = true;
          }
          if (isset($group_settings['items'][$field]['affected_by'])) {
            $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
          }

          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);

            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;

          $fragment = 'applyform/system_select';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_ambitionAndAvailability($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'ambitionAndAvailability';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );


      switch ($field) {
        case 'ambition':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/text';
          break;
        case 'availableIn':
          $table = $group_settings['items'][$field]['system_table'];
          $options = self::get_system_select_options($table);
          $data['options'] = $options;
          $data['table'] = $table;
          if (isset($group_settings['items'][$field]['affects'])) {
            $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
          }
          if (isset($group_settings['items'][$field]['affectable'])) {
            $data['affectable'] = true;
          }
          if (isset($group_settings['items'][$field]['affected_by'])) {
            $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
          }
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);
            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/system_select';


          break;
        case 'availableFrom':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/date';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_transportation($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'transportation';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        case 'homeWork':
        case 'license':
          $table = $group_settings['items'][$field]['system_table'];
          $options = self::get_system_select_options($table);

          $data['options'] = $options;
          $data['table'] = $table;
          if (isset($group_settings['items'][$field]['affects'])) {
            $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
          }
          if (isset($group_settings['items'][$field]['affectable'])) {
            $data['affectable'] = true;
          }
          if (isset($group_settings['items'][$field]['affected_by'])) {
            $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
          }
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);
            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/system_select';

          break;
        case 'haveCar':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          if ($value) {
            $data['checked'] = 'checked';
          }
          $fragment = 'applyform/check';
          break;
        case 'numberValidity':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/date';
          break;
        case 'issuingPlace':
        case 'licenceNo':
        default:
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/text';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_workExperience($group_settings, $visibleFields, $requiredFields, $errors, $values) {

    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'workExperience';
    $repeatClass = 'workExperience';

    $workHistories = new Carerix_Api_Rest_Collection();
    $count = 1;
    if (!empty($values)) {
      foreach ($values as $value => $nr_values) {
        $count = count($nr_values);
        break;
      }
    } else {
      if (self::$employee !== null) {
        $workHistories = self::$employee->getWorkHistories();
        if ($workHistories) {
          $count = (int) $workHistories->getTotalCount();
        }
        if ($count === 0) {
          $count = 1;
        }
      }
    }

    $style = '';
    if ($count <= 1) {
      $style = 'style="display: none;"';
    }
    for ($i = 0; $i < $count; $i++) {
      echo '<div class="' . $repeatClass . '">';

      $workHistory = new Entity_Carerix_WorkExperience($values, $workHistories);
      $workDataValues = $workHistory->getData();
      foreach ($visibleFields as $field) {

        $is_required = array_key_exists($field, $requiredFields);
        $class_chain = 'carerix_input';
        if ($is_required) {
          $class_chain .= ' carerix_required';
          $class_chain .= " $group-$field";
        }

        $fragment = false;

        $error = '';
        if (isset($errors[$field])) {
          $error = $errors[$field];
        }

        $value = '';
        if (isset($workDataValues[$field])) {
          $value = $workDataValues[$field];
        }
        $data = array(
          'label' => @$labels[$group_settings['items'][$field]['label']],
          'name' => "{$group}[{$field}][]",
          'id' => "$group-{$field}",
          'value' => $value,
          'error' => $error,
          'disabled' => false,
          'class_chain' => $class_chain
        );

        switch ($field) {

          case 'periodOf':
          case 'periodUntil':
            $fragment = 'applyform/date';
            break;
          case 'position':
          case 'position_level_1':
          case 'position_level_2':
          case 'industry':
          case 'sector':
            $table = $group_settings['items'][$field]['system_table'];
            $options = self::get_system_select_options($table);

            $data['options'] = $options;
            $data['table'] = $table;
            if (isset($group_settings['items'][$field]['affects'])) {
              $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
            }
            if (isset($group_settings['items'][$field]['affectable'])) {
              $data['affectable'] = true;
            }
            if (isset($group_settings['items'][$field]['affected_by'])) {
              $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
            }

            $fragment = 'applyform/system_select';
            break;
          case 'employer':
          case 'functionName':
          case 'workLocation':
          case 'assignment':
            $fragment = 'applyform/text';
            break;
        }

        if ($fragment) {
          $input_text_fragment = new Utils_Template($fragment);
          $input_text_fragment->with($data)->display();
        }
      }
      echo '<div style="text-align: right;"><a ' . $style . ' href="#" data-repeatclass="' . $repeatClass . '" class="block-derepeat ' . $repeatClass . '"><i class="fa fa-minus"></i> ' . $labels['general.button.remove'] . '</a></div>';
      echo '</div>';
    }

    self::_repeat_html_by_class($repeatClass);
  }

  private static function displaySection_experienceOther($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $group = 'experienceOther';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        default:
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/textarea';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_education($group_settings, $visibleFields, $requiredFields, $errors, $values) {

    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'education';
    $repeatClass = 'education';

    $employeeEducations = new Carerix_Api_Rest_Collection();
    $count = 1;
    if (!empty($values)) {
      foreach ($values as $value => $nr_values) {
        $count = count($nr_values);
        break;
      }
    } else {
      if (self::$employee !== null) {
        $employeeEducations = self::$employee->getEducations();
        if ($employeeEducations) {
          $count = (int) $employeeEducations->getTotalCount();
        }
        if ($count === 0) {
          $count = 1;
        }
      }
    }

    $style = '';
    if ($count <= 1) {
      $style = 'style="display: none;"';
    }

    for ($i = 0; $i < $count; $i++) {
      echo '<div class="' . $repeatClass . '">';

      $education = new Entity_Carerix_EmployeeEducation($values, $employeeEducations);
      $educationDataValues = $education->getData();

      foreach ($visibleFields as $field) {

        $is_required = array_key_exists($field, $requiredFields);
        $class_chain = 'carerix_input';
        if ($is_required) {
          $class_chain .= ' carerix_required';
          $class_chain .= " $group-$field";
        }

        $fragment = false;

        $error = '';
        if (isset($errors[$field])) {
          $error = $errors[$field];
        }

        $value = '';
        if (isset($educationDataValues[$field])) {
          $value = $educationDataValues[$field];
        }
        $data = array(
          'label' => @$labels[$group_settings['items'][$field]['label']],
          'name' => "{$group}[{$field}][]",
          'id' => "$group-{$field}",
          'value' => $value,
          'error' => $error,
          'disabled' => false,
          'class_chain' => $class_chain
        );

        switch ($field) {

          case 'yearStart':
          case 'yearEnd':
          case 'institute':
            $fragment = 'applyform/text';
            break;
          case 'education':
          case 'education_1':
          case 'education_2':
            $table = $group_settings['items'][$field]['system_table'];
            $options = self::get_system_select_options($table);

            $data['options'] = $options;
            $data['table'] = $table;
            if (isset($group_settings['items'][$field]['affects'])) {
              $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
            }
            if (isset($group_settings['items'][$field]['affectable'])) {
              $data['affectable'] = true;
            }
            if (isset($group_settings['items'][$field]['affected_by'])) {
              $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
            }

            $fragment = 'applyform/system_select';
            break;
        }

        if ($fragment) {
          $input_text_fragment = new Utils_Template($fragment);
          $input_text_fragment->with($data)->display();
        }
      }
      echo '<div style="text-align: right;"><a ' . $style . ' href="#" data-repeatclass="' . $repeatClass . '" class="block-derepeat ' . $repeatClass . '"><i class="fa fa-minus"></i> ' . $labels['general.button.remove'] . '</a></div>';
      echo '</div>';
    }

    self::_repeat_html_by_class($repeatClass);
  }

  private static function displaySection_educationOther($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $group = 'educationOther';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        default:
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/textarea';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_professionalSkills($group_settings, $visibleFields, $requiredFields, $errors, $values) {

    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'professionalSkills';
    $repeatClass = 'professionalSkill';

    $employeeSkills = new Carerix_Api_Rest_Collection();
    $count = 1;
    if (!empty($values)) {
      foreach ($values as $value => $nr_values) {
        $count = count($nr_values);
        break;
      }
    } else {
      if (self::$employee !== null) {
        $employeeSkills = self::$employee->getSkills();
        if ($employeeSkills) {
          $count = (int) $employeeSkills->getTotalCount();
        }
        if ($count === 0) {
          $count = 1;
        }
      }
    }

    $style = '';
    if ($count <= 1) {
      $style = 'style="display: none;"';
    }

    for ($i = 0; $i < $count; $i++) {
      echo '<div class="' . $repeatClass . '">';

      $skill = new Entity_Carerix_EmployeeSkill($values, $employeeSkills);
      $skillDataValues = $skill->getData();

      foreach ($visibleFields as $field) {

        $is_required = array_key_exists($field, $requiredFields);
        $class_chain = 'carerix_input';
        if ($is_required) {
          $class_chain .= ' carerix_required';
          $class_chain .= " $group-$field";
        }

        $fragment = false;

        $error = '';
        if (isset($errors[$field])) {
          $error = $errors[$field];
        }

        $value = '';
        if (isset($skillDataValues[$field])) {
          $value = $skillDataValues[$field];
        }
        $data = array(
          'label' => @$labels[$group_settings['items'][$field]['label']],
          'name' => "{$group}[{$field}][]",
          'id' => "$group-{$field}",
          'value' => $value,
          'error' => $error,
          'disabled' => false,
          'class_chain' => $class_chain
        );

        switch ($field) {
          case 'skill':
          case 'skill_1':
          case 'skill_2':
            $table = $group_settings['items'][$field]['system_table'];
            $options = self::get_system_select_options($table);

            $data['options'] = $options;
            $data['table'] = $table;
            if (isset($group_settings['items'][$field]['affects'])) {
              $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
            }
            if (isset($group_settings['items'][$field]['affectable'])) {
              $data['affectable'] = true;
            }
            if (isset($group_settings['items'][$field]['affected_by'])) {
              $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
            }

            $fragment = 'applyform/system_select';
            break;
        }

        if ($fragment) {
          $input_text_fragment = new Utils_Template($fragment);
          $input_text_fragment->with($data)->display();
        }
      }
      echo '<div style="text-align: right;"><a ' . $style . ' href="#" data-repeatclass="' . $repeatClass . '" class="block-derepeat ' . $repeatClass . '"><i class="fa fa-minus"></i> ' . $labels['general.button.remove'] . '</a></div>';
      echo '</div>';
    }

    self::_repeat_html_by_class($repeatClass);
  }

  private static function displaySection_skillsOther($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $group = 'skillsOther';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        default:
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/textarea';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_languages($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'languages';
    $repeatClass = 'language';

    $employeeLanguages = new Carerix_Api_Rest_Collection();
    $count = 1;
    if (!empty($values)) {
      foreach ($values as $value => $nr_values) {
        $count = count($nr_values);
        break;
      }
    } else {
      if (self::$employee !== null) {
        $employeeLanguages = self::$employee->getLanguages();
        if ($employeeLanguages) {
          $count = (int) $employeeLanguages->getTotalCount();
        }
        if ($count === 0) {
          $count = 1;
        }
      }
    }

    $style = '';
    if ($count <= 1) {
      $style = 'style="display: none;"';
    }

    for ($i = 0; $i < $count; $i++) {
      echo '<div class="' . $repeatClass . '">';

      $language = new Entity_Carerix_EmployeeLanguage($values, $employeeLanguages);
      $languageDataValues = $language->getData();

      foreach ($visibleFields as $field) {

        $is_required = array_key_exists($field, $requiredFields);
        $class_chain = 'carerix_input';
        if ($is_required) {
          $class_chain .= ' carerix_required';
          $class_chain .= " $group-$field";
        }

        $fragment = false;

        $error = '';
        if (isset($errors[$field])) {
          $error = $errors[$field];
        }

        $value = '';
        if (isset($languageDataValues[$field])) {
          $value = $languageDataValues[$field];
        }
        $data = array(
          'label' => @$labels[$group_settings['items'][$field]['label']],
          'name' => "{$group}[{$field}][]",
          'id' => "$group-{$field}",
          'value' => $value,
          'error' => $error,
          'disabled' => false,
          'class_chain' => $class_chain
        );

        switch ($field) {
          case 'language':
          case 'read':
          case 'write':
          case 'speak':
            $table = $group_settings['items'][$field]['system_table'];
            $options = self::get_system_select_options($table);

            $data['options'] = $options;
            $data['table'] = $table;
            if (isset($group_settings['items'][$field]['affects'])) {
              $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
            }
            if (isset($group_settings['items'][$field]['affectable'])) {
              $data['affectable'] = true;
            }
            if (isset($group_settings['items'][$field]['affected_by'])) {
              $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
            }

            $fragment = 'applyform/system_select';
            break;
        }

        if ($fragment) {
          $input_text_fragment = new Utils_Template($fragment);
          $input_text_fragment->with($data)->display();
        }
      }
      echo '<div style="text-align: right;"><a ' . $style . ' href="#" data-repeatclass="' . $repeatClass . '" class="block-derepeat ' . $repeatClass . '"><i class="fa fa-minus"></i> ' . $labels['general.button.remove'] . '</a></div>';
      echo '</div>';
    }

    self::_repeat_html_by_class($repeatClass);
  }

  private static function displaySection_languagesOther($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $group = 'languagesOther';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        default:
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/textarea';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_motivationAndSource($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'motivationAndSource';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        case 'source':
          $table = $group_settings['items'][$field]['system_table'];
          $options = self::get_system_select_options($table);

          $data['options'] = $options;
          $data['table'] = $table;
          if (isset($group_settings['items'][$field]['affects'])) {
            $data['affects'] = $group . '-' . $group_settings['items'][$field]['affects'];
          }
          if (isset($group_settings['items'][$field]['affectable'])) {
            $data['affectable'] = true;
          }
          if (isset($group_settings['items'][$field]['affected_by'])) {
            $data['affected_by'] = $group . '-' . $group_settings['items'][$field]['affected_by'];
          }
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'getTo' . ucfirst($mapsTo);
            try {
              $datanode = self::$employee->$method();
              if ($datanode && $datanode->getId()) {
                $value = $datanode->getId();
              }
            } catch (Exception $ex) {
              try {
                $datanode = self::$user->$method();
                if ($datanode) {
                  $value = $datanode->getId();
                }
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/system_select';
          break;
        case 'motivation':
        case 'sourceOther':
          if (!$value && self::$employee) {
            $mapsTo = $group_settings['items'][$field]['mapsTo'];
            $method = 'get' . ucfirst($mapsTo);
            try {
              $value = self::$employee->$method();
            } catch (Exception $ex) {
              try {
                $value = self::$user->$method();
              } catch (Exception $ex) {
                $value = '';
              }
            }
          }
          $data['value'] = $value;
          $fragment = 'applyform/textarea';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  private static function displaySection_newsletterSubscription($group_settings, $visibleFields, $requiredFields, $errors, $values) {
    $labels = Utils_Labels::get();
    $header = $labels[$group_settings['label']];
    echo "<h3 class='cx_h3'>$header</h3>";
    $group = 'newsletterSubscription';
    foreach ($visibleFields as $field) {

      $is_required = array_key_exists($field, $requiredFields);
      $class_chain = 'carerix_input';
      if ($is_required) {
        $class_chain .= ' carerix_required';
        $class_chain .= " $group-$field";
      }

      $fragment = false;

      $error = '';
      if (isset($errors[$field])) {
        $error = $errors[$field];
      }

      $value = '';
      if (isset($values[$field])) {
        $value = $values[$field];
      }
      $data = array(
        'label' => @$labels[$group_settings['items'][$field]['label']],
        'name' => "{$group}[{$field}]",
        'id' => "$group-{$field}",
        'value' => $value,
        'error' => $error,
        'disabled' => false,
        'class_chain' => $class_chain
      );

      switch ($field) {
        case 'subscribe':
          $fragment = 'applyform/check';
          break;
      }

      if ($fragment) {
        $input_text_fragment = new Utils_Template($fragment);
        $input_text_fragment->with($data)->display();
      }
    }
  }

  public static function _repeat_html_by_class($repeatClass, $count = 1) {
    $labels = Utils_Labels::get();
    echo '<div>';
    echo '<a href="#" data-repeatclass="' . $repeatClass . '" class="block-repeat ' . $repeatClass . '"><i class="fa fa-plus"></i> ' . $labels['general.button.add'] . '</a>';
    echo '</div>';
  }
  
  private static function localizeConfirmationPage (&$settings, $interfacer) {
      if ($interfacer->wpml_is_active() && @$settings['page_id']) {
        $page_translation = $interfacer->wpml_get_page_translation($settings['page_id'], $interfacer->language_major_current());

        if ($page_translation) {
            $page_id = $page_translation->element_id;
            $page_permalink = $interfacer->get_post_permalink($page_id);

            $settings['page_id'] = $page_id;
            $settings['page_url'] = $page_permalink;
        }
    }
  }
}
